@@ -5,7 +5,7 @@
 	This software may be used and distributed according to the terms of
 	the GNU General Public License (GPL), incorporated herein by reference.
 	Drivers based on or derived from this code fall under the GPL and must
-	retain the authorship, copyright and license notice.  This file is not
+	retain the authorship, copryight and license notice.  This file is not
 	a complete program and may only be used when the entire operating
 	system is licensed under the GPL.
 
@@ -13,12 +13,12 @@
 	Members of the series include Fast EtherLink 3c590/3c592/3c595/3c597
 	and the EtherLink XL 3c900 and 3c905 cards.
 
-	The author may be reached as becker@scyld.com, or C/O
+	The original author may be reached as becker@scyld.com, or C/O
 	Scyld Computing Corporation
 	410 Severn Ave., Suite 210
 	Annapolis MD 21403
 
-	Support and updates available at
+	Support information and updates are available at
 	http://www.scyld.com/network/vortex.html
 */
 
@@ -35,7 +35,7 @@ static int debug = 1;			/* 1 normal messages, 0 quiet .. 7 verbose. */
 
 /* This driver uses 'options' to pass the media type, full-duplex flag, etc.
    See media_tbl[] and the web page for the possible types.
-   There is no limit card count, MAX_UNITS limits only module options. */
+   There is no limit on card count, MAX_UNITS limits only module options. */
 #define MAX_UNITS 8
 static int options[MAX_UNITS] = { -1, -1, -1, -1, -1, -1, -1, -1,};
 static int full_duplex[MAX_UNITS] = {-1, -1, -1, -1, -1, -1, -1, -1};
@@ -46,9 +46,15 @@ static int max_interrupt_work = 32;
 /* Set the copy breakpoint for the copy-only-tiny-frames scheme.
    Setting to > 1512 effectively disables this feature. */
 static const int rx_copybreak = 200;
-/* Allow setting MTU to a larger size, bypassing the normal ethernet setup. */
+/* Allow setting MTU to a larger size, bypassing the normal Ethernet setup. */
 static const int mtu = 1500;
 
+/* Use hardware checksums? */
+static int use_hw_csums = 1;
+
+/* For Cyclone,Tornado cards, polling interval for DownListPtr */
+static int down_poll_rate = 20;
+
 /* Operational parameters that are set at compile time. */
 
 /* Keep the ring sizes a power of two for compile efficiency.
@@ -62,7 +68,7 @@ static const int mtu = 1500;
 
 /* Operational parameters that usually are not changed. */
 /* Time in jiffies before concluding the transmitter is hung. */
-#define TX_TIMEOUT  ((400*HZ)/1000)
+#define TX_TIMEOUT  (2*HZ)
 
 #define PKT_BUF_SZ		1536			/* Size of each temporary Rx buffer.*/
 
@@ -96,9 +102,9 @@ static const int mtu = 1500;
 #include <linux/etherdevice.h>
 #include <linux/skbuff.h>
 #include <asm/irq.h>
+#include <asm/byteorder.h>
 #include <asm/bitops.h>
 #include <asm/io.h>
-#include <asm/byteorder.h>
 
 /* Kernel compatibility defines, some common to David Hinds' PCMCIA package.
    This is only in the support-all-kernels source code. */
@@ -134,12 +140,14 @@ static const int mtu = 1500;
 #define le32desc_to_virt(addr)  bus_to_virt(le32_to_cpu(addr))
 
 MODULE_AUTHOR("Donald Becker <becker@scyld.com>");
-MODULE_DESCRIPTION("3Com 3c590/3c900 series Vortex/Boomerang driver");
+MODULE_DESCRIPTION("3Com EtherLink XL (3c590/3c900 series) driver");
 MODULE_PARM(debug, "i");
 MODULE_PARM(options, "1-" __MODULE_STRING(MAX_UNITS) "i");
 MODULE_PARM(full_duplex, "1-" __MODULE_STRING(MAX_UNITS) "i");
 MODULE_PARM(rx_copybreak, "i");
 MODULE_PARM(max_interrupt_work, "i");
+MODULE_PARM(use_hw_csums, "i");
+MODULE_PARM(down_poll_rate, "i");
 
 /* Operational parameter that usually are not changed. */
 
@@ -148,7 +156,7 @@ MODULE_PARM(max_interrupt_work, "i");
    code size of a per-interface flag is not worthwhile. */
 static char mii_preamble_required = 0;
 
-/* Values here only for performance evaluation and path-coverage debugging. */
+/* Performance and path-coverage information. */
 static int rx_nocopy = 0, rx_copy = 0, queued_packet = 0, rx_csumhits;
 
 /*
@@ -209,8 +217,8 @@ copying cost of copying a frame to a correctly-sized skbuff.
 
 
 IIIC. Synchronization
-The driver runs as two independent, single-threaded flows of control.
-One is the send-packet routine, which is single threaded by the tx queue
+The driver runs as two independent, single-threaded flows of control.  One
+is the send-packet routine, which enforces single threaded by the tx queue
 system.  The other thread is the interrupt handler, which is single
 threaded by the hardware and other software.
 
@@ -251,7 +259,8 @@ struct pci_id_info {
 #define CYCLONE_SIZE 0x80
 enum { IS_VORTEX=1, IS_BOOMERANG=2, IS_CYCLONE=4, IS_TORNADO=8,
 	   HAS_PWR_CTRL=0x10, HAS_MII=0x20, HAS_NWAY=0x40, HAS_CB_FNS=0x80,
-	   EEPROM_8BIT=0x200, INVERT_LED_PWR=0x400, INVERT_MII_PWR=0x800, };
+	   EEPROM_8BIT=0x200, INVERT_LED_PWR=0x400, MII_XCVR_PWR=0x800,
+	   HAS_FIFO_BUG=0x1000, };
 static struct net_device *vortex_probe1(int pci_bus, int pci_devfn,
 									struct net_device *dev, long ioaddr,
 									int irq, int dev_id, int card_idx);
@@ -262,18 +271,18 @@ static struct pci_id_info pci_tbl[] = {
 	{"3CCFE575BT Cyclone CardBus",	0x10B7, 0x5157, 0xffff,
 	 PCI_USES_IO|PCI_USES_MASTER, IS_CYCLONE|HAS_NWAY|HAS_CB_FNS|
 	 EEPROM_8BIT|INVERT_LED_PWR, 128, vortex_probe1},
-	{"3CCFE575CT Cyclone CardBus",	0x10B7, 0x5257, 0xffff,
-	 PCI_USES_IO|PCI_USES_MASTER, IS_CYCLONE|HAS_NWAY|HAS_CB_FNS|
-	 EEPROM_8BIT|INVERT_MII_PWR, 128, vortex_probe1},
+	{"3CCFE575CT Tornado CardBus",	0x10B7, 0x5257, 0xffff,
+	 PCI_USES_IO|PCI_USES_MASTER, IS_TORNADO|HAS_NWAY|HAS_CB_FNS|
+	 EEPROM_8BIT|MII_XCVR_PWR, 128, vortex_probe1},
 	{"3CCFE656 Cyclone CardBus",	0x10B7, 0x6560, 0xffff,
 	 PCI_USES_IO|PCI_USES_MASTER, IS_CYCLONE|HAS_NWAY|HAS_CB_FNS|
-	 EEPROM_8BIT|INVERT_LED_PWR|INVERT_MII_PWR, 128, vortex_probe1},
+	 EEPROM_8BIT|INVERT_LED_PWR|MII_XCVR_PWR, 128, vortex_probe1},
 	{"3CCFEM656B Cyclone CardBus",	0x10B7, 0x6562, 0xffff,
 	 PCI_USES_IO|PCI_USES_MASTER, IS_CYCLONE|HAS_NWAY|HAS_CB_FNS|
-	 EEPROM_8BIT|INVERT_LED_PWR|INVERT_MII_PWR, 128, vortex_probe1},
-	{"3CCFEM656C Cyclone CardBus",	0x10B7, 0x6564, 0xffff,
-	 PCI_USES_IO|PCI_USES_MASTER, IS_CYCLONE|HAS_NWAY|HAS_CB_FNS|
-	 EEPROM_8BIT|INVERT_MII_PWR, 128, vortex_probe1},
+	 EEPROM_8BIT|INVERT_LED_PWR|MII_XCVR_PWR, 128, vortex_probe1},
+	{"3CXFEM656C Tornado CardBus",	0x10B7, 0x6564, 0xffff,
+	 PCI_USES_IO|PCI_USES_MASTER, IS_TORNADO|HAS_NWAY|HAS_CB_FNS|
+	 EEPROM_8BIT|MII_XCVR_PWR, 128, vortex_probe1},
 	{"3c575 series CardBus (unknown version)", 0x10B7, 0x5057, 0xf0ff,
 	 PCI_USES_IO|PCI_USES_MASTER, IS_BOOMERANG|HAS_MII|
 	 EEPROM_8BIT|INVERT_LED_PWR, 64, vortex_probe1},
@@ -307,7 +316,8 @@ enum vortex_cmd {
 	SetStatusEnb = 15<<11, SetRxFilter = 16<<11, SetRxThreshold = 17<<11,
 	SetTxThreshold = 18<<11, SetTxStart = 19<<11,
 	StartDMAUp = 20<<11, StartDMADown = (20<<11)+1, StatsEnable = 21<<11,
-	StatsDisable = 22<<11, StopCoax = 23<<11, SetFilterBit = 25<<11,};
+	StatsDisable = 22<<11, StopCoax = 23<<11, SetTxReclaim = 24<<11,
+	SetFilterBit = 25<<11,};
 
 /* The SetRxFilter command accepts the following classes: */
 enum RxFilter {
@@ -378,7 +388,7 @@ enum Window7 {					/* Window 7: Bus Master control. */
 /* Boomerang bus master control registers. */
 enum MasterCtrl {
 	PktStatus = 0x20, DownListPtr = 0x24, FragAddr = 0x28, FragLen = 0x2c,
-	DownPollRate = 0x2d, TxFreeThreshold = 0x2f,
+	DownBurstThresh=0x2a, DownPollRate = 0x2d, TxFreeThreshold = 0x2f,
 	UpPktStatus = 0x30, UpListPtr = 0x38,
 };
 
@@ -427,12 +437,15 @@ struct vortex_private {
 	struct sk_buff* tx_skbuff[TX_RING_SIZE];
 	struct net_device *next_module;
 	void *priv_addr;
+	/* Keep the Rx and Tx variables grouped on their own cache lines. */
 	struct boom_rx_desc *rx_head_desc;
 	unsigned int cur_rx, dirty_rx;		/* Producer/consumer ring indices */
 	unsigned int rx_buf_sz;				/* Based on MTU+slack. */
+
 	struct boom_tx_desc *tx_desc_tail;
 	unsigned int cur_tx, dirty_tx;
 	struct sk_buff *tx_skb;		/* Packet being eaten by bus master ctrl.  */
+
 	long last_reset;
 	struct net_device_stats stats;
 	char *cb_fn_base;			/* CardBus function status addr space. */
@@ -444,7 +457,7 @@ struct vortex_private {
 	int options;				/* User-settable misc. driver options. */
 	unsigned int media_override:4, 			/* Passed-in media type. */
 		default_media:4,				/* Read from the EEPROM/Wn3_Config. */
-		full_duplex:1, force_fd:1, autoselect:1,
+		full_duplex:1, medialock:1, autoselect:1,
 		bus_master:1,				/* Vortex can only do a fragment bus-m. */
 		full_bus_master_tx:1, full_bus_master_rx:2, /* Boomerang  */
 		hw_csums:1,				/* Has hardware checksums. */
@@ -458,6 +471,7 @@ struct vortex_private {
 	u16 advertising;					/* NWay media advertisement */
 	unsigned char phys[2];				/* MII device addresses. */
 	u16 deferred_irqs;
+	spinlock_t lock;
 };
 
 /* The action to take with a media selection timer tick.
@@ -492,6 +506,7 @@ static void vortex_up(struct net_device *dev);
 static void vortex_down(struct net_device *dev);
 static int vortex_open(struct net_device *dev);
 static void set_media_type(struct net_device *dev);
+static void activate_xcvr(struct net_device *dev);
 static void start_operation(struct net_device *dev);
 static void mdio_sync(long ioaddr, int bits);
 static int mdio_read(long ioaddr, int phy_id, int location);
@@ -647,7 +662,7 @@ static struct net_device *vortex_probe1(int pci_bus, int pci_devfn,
 	int option;
 	unsigned int eeprom[0x40], checksum = 0;		/* EEPROM contents */
 	int drv_flags = pci_tbl[chip_idx].drv_flags;
-	int i;
+	int i, step;
 
 	dev = init_etherdev(dev, 0);
 
@@ -698,7 +713,8 @@ static struct net_device *vortex_probe1(int pci_bus, int pci_devfn,
 	if (find_cnt < MAX_UNITS  &&  full_duplex[find_cnt] > 0)
 		vp->full_duplex = 1;
 
-	vp->force_fd = vp->full_duplex;
+	if (vp->full_duplex)
+		vp->medialock = 1;
 	vp->options = option;
 
 	/* Read the station address from the EEPROM. */
@@ -707,7 +723,7 @@ static struct net_device *vortex_probe1(int pci_bus, int pci_devfn,
 		int timer;
 		outw((drv_flags & EEPROM_8BIT ? 0x230 : EEPROM_Read) + i,
 			 ioaddr + Wn0EepromCmd);
-		/* The read may take up to 162 usec. */
+		/* The read may take up to 162 usec, timed with PCI cycles. */
 		for (timer = 162*5; timer >= 0; timer--) {
 			if ((inw(ioaddr + Wn0EepromCmd) & 0x8000) == 0)
 				break;
@@ -737,15 +753,19 @@ static struct net_device *vortex_probe1(int pci_bus, int pci_devfn,
 	printk(", IRQ %s\n", __irq_itoa(dev->irq));
 #else
 	printk(", IRQ %d\n", dev->irq);
-	printk(KERN_INFO "  product code '%c%c' rev %02x date %02d-%02d-%02d\n",
-		   eeprom[6]&0xff, eeprom[6]>>8, eeprom[0x14],
-		   (eeprom[4]>>5) & 15, eeprom[4] & 31, eeprom[4]>>9);
 	/* Tell them about an invalid IRQ. */
 	if (vortex_debug && (dev->irq <= 0))
 		printk(KERN_WARNING " *** Warning: IRQ %d is unlikely to work! ***\n",
 			   dev->irq);
 #endif
+	EL3WINDOW(4);
+	step = (inb(ioaddr + Wn4_NetDiag) & 0x1e) >> 1;
+	printk(KERN_INFO "  product code '%c%c' rev %02x.%d date %02d-"
+		   "%02d-%02d\n", eeprom[6]&0xff, eeprom[6]>>8, eeprom[0x14],
+		   step, (eeprom[4]>>5) & 15, eeprom[4] & 31, eeprom[4]>>9);
 
+	if ((eeprom[0x14] == 0x07) && (step == 1))
+		vp->drv_flags = drv_flags |= HAS_FIFO_BUG;
 	if (drv_flags & HAS_CB_FNS) {
 		u32 fn_st_addr;			/* Cardbus function status space */
 		pcibios_read_config_dword(pci_bus, pci_devfn, PCI_BASE_ADDRESS_2,
@@ -754,11 +774,6 @@ static struct net_device *vortex_probe1(int pci_bus, int pci_devfn,
 			vp->cb_fn_base = ioremap(fn_st_addr & ~3, 128);
 		printk(KERN_INFO "%s: CardBus functions mapped %8.8x->%p.\n",
 			   dev->name, fn_st_addr, vp->cb_fn_base);
-		if (drv_flags & INVERT_MII_PWR) {
-			EL3WINDOW(2);
-			outw(inw(ioaddr + Wn2_ResetOptions) | 0x4000,
-				 ioaddr + Wn2_ResetOptions);
-		}
 	}
 
 	/* Extract our information from the EEPROM data. */
@@ -769,6 +784,9 @@ static struct net_device *vortex_probe1(int pci_bus, int pci_devfn,
 	if (vp->info1 & 0x8000)
 		vp->full_duplex = 1;
 
+	/* Turn on the transceiver. */
+	activate_xcvr(dev);
+
 	{
 		char *ram_split[] = {"5:3", "3:1", "1:1", "3:5"};
 		union wn3_config config;
@@ -876,21 +894,11 @@ vortex_up(struct net_device *dev)
 	struct vortex_private *vp = (struct vortex_private *)dev->priv;
 	long ioaddr = dev->base_addr;
 	int i;
-	
+
 	acpi_wake(vp->pci_bus, vp->pci_devfn);
 
-	if (vp->cb_fn_base) {
-		u16 opts;
-		EL3WINDOW(2);
-		opts = inw(ioaddr + Wn2_ResetOptions);
-		/* Inverted LED polarity */
-		if (vp->drv_flags & INVERT_LED_PWR)
-			opts |= 0x0010;
-		/* Inverted polarity of MII power bit */
-		if (vp->drv_flags & INVERT_MII_PWR)
-			opts |= 0x4000;
-		outw(opts, ioaddr + Wn2_ResetOptions);
-	}
+	vp->lock = SPIN_LOCK_UNLOCKED;
+	activate_xcvr(dev);
 
 	/* Before initializing select the active media port. */
 	if (vp->media_override != 7) {
@@ -911,7 +919,8 @@ vortex_up(struct net_device *dev)
 	} else
 		dev->if_port = vp->default_media;
 
-	vp->full_duplex = vp->force_fd;
+	if (! vp->medialock)
+		vp->full_duplex = 0;
 
 	vp->status_enable = SetStatusEnb | HostError|IntReq|StatsFull|TxComplete|
 		(vp->full_bus_master_tx ? DownComplete : TxAvailable) |
@@ -929,6 +938,8 @@ vortex_up(struct net_device *dev)
 	set_media_type(dev);
 	start_operation(dev);
 
+	spin_lock(&vp->lock);
+
 	if (vortex_debug > 1) {
 		EL3WINDOW(4);
 		printk(KERN_DEBUG "%s: vortex_open() irq %d media status %4.4x.\n",
@@ -950,7 +961,9 @@ vortex_up(struct net_device *dev)
 
 	/* Switch to register set 7 for normal use. */
 	EL3WINDOW(7);
+	spin_unlock(&vp->lock);
 
+	vp->tx_full = 0;
 	if (vp->full_bus_master_rx) { /* Boomerang bus master. */
 		vp->cur_rx = vp->dirty_rx = 0;
 		/* Initialize the RxEarly register as recommended. */
@@ -964,20 +977,23 @@ vortex_up(struct net_device *dev)
 	if (vp->full_bus_master_tx) { 		/* Boomerang bus master Tx. */
 		vp->cur_tx = vp->dirty_tx = 0;
 		vp->tx_desc_tail = &vp->tx_ring[TX_RING_SIZE - 1];
-		if (vp->drv_flags & IS_BOOMERANG) {
+		if ((vp->drv_flags & IS_BOOMERANG) || !down_poll_rate) {
 			/* Room for a packet, to avoid long DownStall delays. */
 			outb(PKT_BUF_SZ>>8, ioaddr + TxFreeThreshold);
 		} else
-			outb(20, ioaddr + DownPollRate);
+			outb(down_poll_rate, ioaddr + DownPollRate);
+
 		/* Clear the Tx ring. */
 		for (i = 0; i < TX_RING_SIZE; i++)
 			vp->tx_skbuff[i] = 0;
 		outl(0, ioaddr + DownListPtr);
 		vp->restart_tx = 1;
 	}
-	/* Set receiver mode: presumably accept b-case and phys addr only. */
+	/* Set receiver mode: presumably accept b-cast and phys addr only. */
 	set_rx_mode(dev);
 	outw(StatsEnable, ioaddr + EL3_CMD); /* Turn on statistics. */
+	if (vp->drv_flags & HAS_FIFO_BUG) /* max out the tx fifo threshold */
+		outb(0x1f, ioaddr + DownBurstThresh);
 
 	outw(RxEnable, ioaddr + EL3_CMD); /* Enable the receiver. */
 	outw(TxEnable, ioaddr + EL3_CMD); /* Enable transmitter. */
@@ -1040,6 +1056,22 @@ static void set_media_type(struct net_device *dev)
 		 (dev->mtu > 1500 ? 0x40 : 0), ioaddr + Wn3_MAC_Ctrl);
 }
 
+static void activate_xcvr(struct net_device *dev)
+{
+	struct vortex_private *vp = (struct vortex_private *)dev->priv;
+	long ioaddr = dev->base_addr;
+	int reset_opts;
+
+	/* Correct some magic bits. */
+	EL3WINDOW(2);
+	reset_opts = inw(ioaddr + Wn2_ResetOptions);
+	if (vp->drv_flags & INVERT_LED_PWR)
+		reset_opts |= 0x0010;
+	if (vp->drv_flags & MII_XCVR_PWR)
+		reset_opts |= 0x4000;
+	outw(reset_opts, ioaddr + Wn2_ResetOptions);
+}
+
 static void start_operation(struct net_device *dev)
 {
 	long ioaddr = dev->base_addr;
@@ -1110,6 +1142,7 @@ static void vortex_timer(unsigned long data)
 	int next_tick = 60*HZ;
 	int ok = 0;
 	int media_status, mii_status, old_window;
+	unsigned long flags;
 
 	if (vortex_debug > 1)
 		printk(KERN_DEBUG "%s: Media selection timer tick happened, %s.\n",
@@ -1151,40 +1184,42 @@ static void vortex_timer(unsigned long data)
 				printk(KERN_DEBUG "%s: Media %s has link beat, %x.\n",
 					   dev->name, media_tbl[dev->if_port].name, media_status);
 		} else if (vortex_debug > 1)
-			printk(KERN_DEBUG "%s: Media %s is has no link beat, %x.\n",
+			printk(KERN_DEBUG "%s: Media %s has no link beat, %x.\n",
 				   dev->name, media_tbl[dev->if_port].name, media_status);
 		break;
 	case XCVR_MII: case XCVR_NWAY:
-		  mii_status = mdio_read(ioaddr, vp->phys[0], 1);
-		  ok = 1;
-		  if (debug > 1)
-			  printk(KERN_DEBUG "%s: MII transceiver has status %4.4x.\n",
-					 dev->name, mii_status);
-		  if (mii_status & 0x0004) {
-			  int mii_reg5 = mdio_read(ioaddr, vp->phys[0], 5);
-			  if (! vp->force_fd  &&  mii_reg5 != 0xffff) {
-				  int duplex = (mii_reg5&0x0100) ||
-					  (mii_reg5 & 0x01C0) == 0x0040;
-				  if (vp->full_duplex != duplex) {
-					  vp->full_duplex = duplex;
-					  printk(KERN_INFO "%s: Setting %s-duplex based on MII "
-							 "#%d link partner capability of %4.4x.\n",
-							 dev->name, vp->full_duplex ? "full" : "half",
-							 vp->phys[0], mii_reg5);
-					  /* Set the full-duplex bit. */
-					  EL3WINDOW(3);
-					  outb((vp->full_duplex ? 0x20 : 0) |
-						   (dev->mtu > 1500 ? 0x40 : 0),
-						   ioaddr + Wn3_MAC_Ctrl);
-				  }
-				  next_tick = 60*HZ;
-			  }
-		  }
-		  break;
-	  default:					/* Other media types handled by Tx timeouts. */
+		spin_lock_irqsave(&vp->lock, flags);
+		mii_status = mdio_read(ioaddr, vp->phys[0], 1);
+		ok = 1;
+		if (debug > 1)
+			printk(KERN_DEBUG "%s: MII transceiver has status %4.4x.\n",
+				   dev->name, mii_status);
+		if (mii_status & 0x0004) {
+			int mii_reg5 = mdio_read(ioaddr, vp->phys[0], 5);
+			if (! vp->medialock  &&  mii_reg5 != 0xffff) {
+				int duplex = (mii_reg5&0x0100) ||
+					(mii_reg5 & 0x01C0) == 0x0040;
+				if (vp->full_duplex != duplex) {
+					vp->full_duplex = duplex;
+					printk(KERN_INFO "%s: Setting %s-duplex based on MII "
+						   "#%d link partner capability of %4.4x.\n",
+						   dev->name, vp->full_duplex ? "full" : "half",
+						   vp->phys[0], mii_reg5);
+					/* Set the full-duplex bit. */
+					EL3WINDOW(3);
+					outb((vp->full_duplex ? 0x20 : 0) |
+						 (dev->mtu > 1500 ? 0x40 : 0),
+						 ioaddr + Wn3_MAC_Ctrl);
+				}
+				next_tick = 60*HZ;
+			}
+		}
+		spin_unlock_irqrestore(&vp->lock, flags);
+		break;
+	default:					/* Other media types handled by Tx timeouts. */
 		if (vortex_debug > 1)
-		  printk(KERN_DEBUG "%s: Media %s is has no indication, %x.\n",
-				 dev->name, media_tbl[dev->if_port].name, media_status);
+			printk(KERN_DEBUG "%s: Media %s has no indication, %x.\n",
+				   dev->name, media_tbl[dev->if_port].name, media_status);
 		ok = 1;
 	}
 	if ( ! ok) {
@@ -1239,8 +1274,8 @@ static void vortex_tx_timeout(struct net_device *dev)
 	printk(KERN_ERR "%s: transmit timed out, tx_status %2.2x status %4.4x.\n",
 		   dev->name, inb(ioaddr + TxStatus), inw(ioaddr + EL3_STATUS));
 	EL3WINDOW(4);
-	printk(KERN_ERR "  diagnostics: fifo %04x net %04x dma %8.8x.\n",
-		   inw(ioaddr + Wn4_FIFODiag), inw(ioaddr + Wn4_NetDiag),
+	printk(KERN_ERR "  diagnostics: net %04x media %04x dma %8.8x.\n",
+		   inw(ioaddr + Wn4_NetDiag), inw(ioaddr + Wn4_Media),
 		   inl(ioaddr + PktStatus));
 	/* Slight code bloat to be user friendly. */
 	if ((inb(ioaddr + TxStatus) & 0x88) == 0x88)
@@ -1249,16 +1284,14 @@ static void vortex_tx_timeout(struct net_device *dev)
 	if (inw(ioaddr + EL3_STATUS) & IntLatch) {
 		printk(KERN_ERR "%s: Interrupt posted but not delivered --"
 			   " IRQ blocked by another device?\n", dev->name);
-		/* Bad idea here.. but we might as well handle a few events. */
-		vortex_interrupt(dev->irq, dev, 0);
 	}
 
 #if ! defined(final_version)
 	if (vp->full_bus_master_tx) {
 		int i;
 		printk(KERN_DEBUG "  Flags; bus-master %d, full %d; dirty %d "
-			   "current %d.\n",
-			   vp->full_bus_master_tx, vp->tx_full, vp->dirty_tx, vp->cur_tx);
+			   "current %d.\n", vp->full_bus_master_tx, vp->tx_full,
+			   vp->dirty_tx % TX_RING_SIZE, vp->cur_tx % TX_RING_SIZE);
 		printk(KERN_DEBUG "  Transmit list %8.8x vs. %p.\n",
 			   inl(ioaddr + DownListPtr),
 			   &vp->tx_ring[vp->dirty_tx % TX_RING_SIZE]);
@@ -1277,6 +1310,8 @@ static void vortex_tx_timeout(struct net_device *dev)
 		if (vortex_debug > 0)
 			printk(KERN_DEBUG "%s: Resetting the Tx ring pointer.\n",
 				   dev->name);
+		if (vp->drv_flags & HAS_FIFO_BUG) /* Disable tx reclaim */
+			outw(SetTxReclaim | 0xff, ioaddr + EL3_CMD);
 		if (vp->cur_tx - vp->dirty_tx > 0  &&  inl(ioaddr + DownListPtr) == 0)
 			outl(virt_to_bus(&vp->tx_ring[vp->dirty_tx % TX_RING_SIZE]),
 				 ioaddr + DownListPtr);
@@ -1284,17 +1319,20 @@ static void vortex_tx_timeout(struct net_device *dev)
 			vp->tx_full = 0;
 			netif_start_queue(dev);
 		}
-		if (vp->drv_flags & IS_BOOMERANG) {
+		if ((vp->drv_flags & IS_BOOMERANG) || !down_poll_rate) {
 			/* Room for a packet, to avoid long DownStall delays. */
 			outb(PKT_BUF_SZ>>8, ioaddr + TxFreeThreshold);
 			outw(DownUnstall, ioaddr + EL3_CMD);
 		} else
-			outb(20, ioaddr + DownPollRate);
-	} else
+			outb(down_poll_rate, ioaddr + DownPollRate);
+	} else {
+		netif_start_queue(dev);
 		vp->stats.tx_dropped++;
+	}
 	
 	/* Issue Tx Enable */
 	outw(TxEnable, ioaddr + EL3_CMD);
+	vp->restart_tx = 1;
 	dev->trans_start = jiffies;
 	
 	/* Switch to register set 7 for normal use. */
@@ -1317,15 +1355,19 @@ vortex_error(struct net_device *dev, int status)
 		/* Presumably a tx-timeout. We must merely re-enable. */
 		if (vortex_debug > 2
 			|| (tx_status != 0x88 && vortex_debug > 0))
-			printk(KERN_DEBUG"%s: Transmit error, Tx status register %2.2x.\n",
+			printk(KERN_INFO "%s: Transmit error, Tx status register %2.2x.\n",
 				   dev->name, tx_status);
 		if (tx_status & 0x14)  vp->stats.tx_fifo_errors++;
 		if (tx_status & 0x38)  vp->stats.tx_aborted_errors++;
 		outb(0, ioaddr + TxStatus);
+		if ((tx_status & 0x08) && (vp->drv_flags & IS_TORNADO))
+			wait_for_completion(dev, TxReset | 0x0108);
 		if (tx_status & 0x30)
 			do_tx_reset = 1;
-		else					/* Merely re-enable the transmitter. */
+		else {					/* Merely re-enable the transmitter. */
 			outw(TxEnable, ioaddr + EL3_CMD);
+			vp->restart_tx = 1;
+		}
 	}
 	if (status & RxEarly) {				/* Rx early is unused. */
 		vortex_rx(dev);
@@ -1348,7 +1390,8 @@ vortex_error(struct net_device *dev, int status)
 			DoneDidThat++;
 		}
 	}
-	if (status & IntReq) {		/* Restore all interrupt sources.  */
+	if (status & IntReq) {
+		/* Restore all interrupt sources.  */
 		outw(vp->status_enable, ioaddr + EL3_CMD);
 		outw(vp->intr_enable, ioaddr + EL3_CMD);
 	}
@@ -1377,8 +1420,16 @@ vortex_error(struct net_device *dev, int status)
 		}
 	}
 	if (do_tx_reset) {
+		wait_for_completion(dev, DownStall);
 		wait_for_completion(dev, TxReset);
 		outw(TxEnable, ioaddr + EL3_CMD);
+		if ((vp->drv_flags & IS_BOOMERANG) || !down_poll_rate) {
+			/* Room for a packet, to avoid long DownStall delays. */
+			outb(PKT_BUF_SZ>>8, ioaddr + TxFreeThreshold);
+			outw(DownUnstall, ioaddr + EL3_CMD);
+		} else
+			outb(down_poll_rate, ioaddr + DownPollRate);
+		vp->restart_tx = 1;
 	}
 
 }
@@ -1430,6 +1481,7 @@ vortex_start_xmit(struct sk_buff *skb, struct net_device *dev)
 					wait_for_completion(dev, TxReset);
 				}
 				outw(TxEnable, ioaddr + EL3_CMD);
+				vp->restart_tx = 1;
 			}
 			outb(0x00, ioaddr + TxStatus); /* Pop the status stack. */
 		}
@@ -1469,9 +1521,8 @@ boomerang_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	vp->tx_ring[entry].length = cpu_to_le32(skb->len | LAST_FRAG);
 	vp->tx_ring[entry].status = cpu_to_le32(skb->len | TxIntrUploaded);
 
-	if (vp->drv_flags & IS_BOOMERANG) {
-		save_flags(flags);
-		cli();
+	spin_lock_irqsave(&vp->lock, flags);
+	if ((vp->drv_flags & IS_BOOMERANG) || !down_poll_rate) {
 		/* Wait for the stall to complete. */
 		wait_for_completion(dev, DownStall);
 		vp->tx_desc_tail->next = virt_to_le32desc(&vp->tx_ring[entry]);
@@ -1481,12 +1532,14 @@ boomerang_start_xmit(struct sk_buff *skb, struct net_device *dev)
 			queued_packet++;
 		}
 		outw(DownUnstall, ioaddr + EL3_CMD);
-		restore_flags(flags);
 	} else {
 		vp->tx_desc_tail->next = virt_to_le32desc(&vp->tx_ring[entry]);
 		vp->tx_desc_tail = &vp->tx_ring[entry];
 		if (vp->restart_tx) {
-			outl(virt_to_bus(vp->tx_desc_tail), ioaddr + DownListPtr);
+			if (vp->drv_flags & HAS_FIFO_BUG) /* Disable tx reclaim */
+				outw(SetTxReclaim | 0xff, ioaddr + EL3_CMD);
+			outl(virt_to_bus(&vp->tx_ring[vp->dirty_tx % TX_RING_SIZE]),
+				 ioaddr + DownListPtr);
 			vp->restart_tx = 0;
 			queued_packet++;
 		}
@@ -1500,6 +1553,7 @@ boomerang_start_xmit(struct sk_buff *skb, struct net_device *dev)
 #endif
 		netif_start_queue(dev);
 	}
+	spin_unlock_irqrestore(&vp->lock, flags);
 	dev->trans_start = jiffies;
 	return 0;
 }
@@ -1527,6 +1581,7 @@ static void vortex_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	if (vortex_debug > 4)
 		printk(KERN_DEBUG "%s: interrupt, status %4.4x, latency %d ticks.\n",
 			   dev->name, status, latency);
+	spin_lock(&vp->lock);
 	do {
 		if (vortex_debug > 5)
 				printk(KERN_DEBUG "%s: In interrupt loop, status %4.4x.\n",
@@ -1586,7 +1641,7 @@ static void vortex_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		}
 
 		if (--work_done < 0) {
-			printk(KERN_WARNING "%s: Too much work in interrupt, status "
+			printk(KERN_DEBUG "%s: Too much work in interrupt, status "
 				   "%4.4x.\n", dev->name, status);
 			/* Disable all pending interrupts. */
 			do {
@@ -1607,6 +1662,7 @@ static void vortex_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 			writel(0x8000, vp->cb_fn_base + 4);
 
 	} while ((status = inw(ioaddr + EL3_STATUS)) & (IntLatch | RxComplete));
+	spin_unlock(&vp->lock);
 
 	if (vortex_debug > 4)
 		printk(KERN_DEBUG "%s: exiting interrupt, status %4.4x.\n",
@@ -1725,8 +1781,7 @@ boomerang_rx(struct net_device *dev)
 				skb_reserve(skb, 2);	/* Align IP on 16 byte boundaries */
 				/* 'skb_put()' points to the start of sk_buff data area. */
 				memcpy(skb_put(skb, pkt_len),
-					   le32desc_to_virt(vp->rx_ring[entry].addr),
-					   pkt_len);
+					   le32desc_to_virt(vp->rx_ring[entry].addr), pkt_len);
 				rx_copy++;
 			} else {
 				void *temp;
@@ -1743,7 +1798,7 @@ boomerang_rx(struct net_device *dev)
 				rx_nocopy++;
 			}
 			skb->protocol = eth_type_trans(skb, dev);
-			{					/* Use hardware checksum info. */
+			if (use_hw_csums) {
 				int csum_bits = rx_status & 0xee000000;
 				if (csum_bits &&
 					(csum_bits == (IPChksumValid | TCPChksumValid) ||
@@ -1860,10 +1915,9 @@ static struct net_device_stats *vortex_get_stats(struct net_device *dev)
 	unsigned long flags;
 
 	if (netif_device_present(dev)) {
-		save_flags(flags);
-		cli();
+		spin_lock_irqsave(&vp->lock, flags);
 		update_stats(dev->base_addr, dev);
-		restore_flags(flags);
+		spin_unlock_irqrestore(&vp->lock, flags);
 	}
 	return &vp->stats;
 }
@@ -1928,6 +1982,9 @@ static int vortex_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 	long ioaddr = dev->base_addr;
 	u16 *data = (u16 *)&rq->ifr_data;
 	int phy = vp->phys[0] & 0x1f;
+	int retval;
+
+	spin_lock(&vp->lock);
 
 	switch(cmd) {
 	case SIOCDEVPRIVATE:		/* Get the address of the PHY in use. */
@@ -1935,16 +1992,36 @@ static int vortex_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 	case SIOCDEVPRIVATE+1:		/* Read the specified MII register. */
 		EL3WINDOW(4);
 		data[3] = mdio_read(ioaddr, data[0] & 0x1f, data[1] & 0x1f);
-		return 0;
+		retval = 0;
+		break;
 	case SIOCDEVPRIVATE+2:		/* Write the specified MII register */
-		if (!capable(CAP_NET_ADMIN))
-			return -EPERM;
-		EL3WINDOW(4);
-		mdio_write(ioaddr, data[0] & 0x1f, data[1] & 0x1f, data[2]);
-		return 0;
+		if (!capable(CAP_NET_ADMIN)) {
+			retval = -EPERM;
+		} else {
+			if (data[0] == vp->phys[0]) {
+				u16 value = data[2];
+				switch (data[1]) {
+				case 0:
+					/* Check for autonegotiation on or reset. */
+					vp->medialock = (value & 0x9000) ? 0 : 1;
+					if (vp->medialock)
+						vp->full_duplex = (value & 0x0100) ? 1 : 0;
+					break;
+				case 4: vp->advertising = value; break;
+				}
+				/* Perhaps check_duplex(dev), depending on chip semantics. */
+			}
+			EL3WINDOW(4);
+			mdio_write(ioaddr, data[0] & 0x1f, data[1] & 0x1f, data[2]);
+			retval = 0;
+		}
+		break;
 	default:
-		return -EOPNOTSUPP;
+		retval = -EOPNOTSUPP;
 	}
+
+	spin_unlock(&vp->lock);
+	return retval;
 }
 
 /* Pre-Cyclone chips have no documented multicast filter, so the only
@@ -2132,9 +2209,6 @@ void cleanup_module(void)
 
 /*
  * Local variables:
- *  compile-command: "gcc -DMODULE -D__KERNEL__ -Wall -Wstrict-prototypes -O6 -c 3c59x.c `[ -f /usr/include/linux/modversions.h ] && echo -DMODVERSIONS`"
- *  SMP-compile-command: "gcc -D__SMP__ -DMODULE -D__KERNEL__ -Wall -Wstrict-prototypes -O6 -c 3c59x.c"
- *  cardbus-compile-command: "gcc -DCARDBUS -DMODULE -D__KERNEL__ -Wall -Wstrict-prototypes -O6 -c 3c59x.c -o 3c575_cb.o -I/usr/src/linux/pcmcia-cs-3.0.9/include/"
  *  c-indent-level: 4
  *  c-basic-offset: 4
  *  tab-width: 4
