@@ -2,7 +2,7 @@
 
     A simple MTD for Intel Series 2 and Series 100 Flash devices
 
-    iflash2_mtd.c 1.55 2000/05/16 21:31:36
+    iflash2_mtd.c 1.58 2000/10/02 20:38:23
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,7 +15,7 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    <dahinds@users.sourceforge.net>.  Portions created by David A. Hinds
     are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
 
     Alternatively, the contents of this file may be used under the
@@ -73,7 +73,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) do { if (pc_debug>(n)) printk(KERN_INFO args); } while (0)
 static char *version =
-"iflash2_mtd.c 1.55 2000/05/16 21:31:36 (David Hinds)";
+"iflash2_mtd.c 1.58 2000/10/02 20:38:23 (David Hinds)";
 #else
 #define DEBUG(n, args...) do { } while (0)
 #endif
@@ -305,7 +305,7 @@ static void reset_block(volatile u_short *address)
 	writew(IF_READ_CSR, address);
 	CSR = readw(address);
 	if (CSR != 0xffff) break;
-	udelay(1000);
+	mdelay(1);
     }
 #ifdef PCMCIA_DEBUG
     if (i)
@@ -836,7 +836,7 @@ done:
     
 ======================================================================*/
 
-static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
+static int flash_erase(dev_link_t *link, mtd_request_t *req)
 {
     flash_dev_t *dev = (flash_dev_t *)link->priv;
     cs_status_t status;
@@ -961,7 +961,7 @@ static int flash_request(dev_link_t *link, void *buf, mtd_request_t *req)
 	ret = flash_write(link, buf, req);
 	break;
     case MTD_REQ_ERASE:
-	ret = flash_erase(link, buf, req);
+	ret = flash_erase(link, req);
 	break;
     case MTD_REQ_COPY:
 	ret = CS_UNSUPPORTED_FUNCTION;
