@@ -28,7 +28,7 @@ References
   Dean Siasoyco, New Media Corporation
   Ken Lesniak, Silicon Graphics, Inc. <lesniak@boston.sgi.com>
   Donald Becker <becker@cesdis1.gsfc.nasa.gov>
-  David Hinds <dhinds@pcmcia.sourceforge.org>
+  David Hinds <dahinds@users.sourceforge.net>
 
   The Linux client driver is based on the 3c589_cs.c client driver by
   David Hinds.
