@@ -89,16 +89,10 @@
 #include <pcmcia/cisreg.h>
 #include <pcmcia/ciscode.h>
 
-#ifndef MANFID_XIRCOM
-  #define MANFID_XIRCOM 	   0x0105
-#endif
 #ifndef MANFID_COMPAQ
   #define MANFID_COMPAQ 	   0x0138
   #define MANFID_COMPAQ2	   0x0183  /* is this correct? */
 #endif
-#ifndef MANFID_INTEL
-  #define MANFID_INTEL		   0x0089
-#endif
 
 #include <pcmcia/ds.h>
 
@@ -256,25 +250,16 @@ static char *version =
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
-static int if_port = 0;
-MODULE_PARM(if_port, "i");
 
-/* Bit map of interrupts to choose from */
-/* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
-static u_long irq_mask = 0xdeb8;
-MODULE_PARM(irq_mask, "i");
+#define INT_MODULE_PARM(n, v) static int n = v; MODULE_PARM(n, "i")
 
 static int irq_list[4] = { -1 };
 MODULE_PARM(irq_list, "1-4i");
-
-static int do_sound = 1;
-MODULE_PARM(do_sound, "i");
-
-static int card_type = 0;
-MODULE_PARM(card_type, "i");  /* dummy, not used anymore */
-
-static int lockup_hack = 0;
-MODULE_PARM(lockup_hack, "i");  /* anti lockup hack */
+INT_MODULE_PARM(irq_mask,	0xdeb8);
+INT_MODULE_PARM(if_port,	0);
+INT_MODULE_PARM(full_duplex,	0);
+INT_MODULE_PARM(do_sound, 	1);
+INT_MODULE_PARM(lockup_hack,	0);  /* anti lockup hack */
 
 /*====================================================================*/
 
@@ -369,7 +354,7 @@ typedef struct local_info_t {
     dev_link_t link;
     struct net_device dev;
     dev_node_t node;
-    struct enet_statistics stats;
+    struct net_device_stats stats;
     int card_type;
     int probe_port;
     int silicon; /* silicon revision. 0=old CE2, 1=Scipper, 4=Mohawk */
@@ -387,7 +372,7 @@ typedef struct local_info_t {
  */
 static int do_start_xmit(struct sk_buff *skb, struct net_device *dev);
 static void do_tx_timeout(struct net_device *dev);
-static struct enet_statistics *do_get_stats(struct net_device *dev);
+static struct net_device_stats *do_get_stats(struct net_device *dev);
 static void set_addresses(struct net_device *dev);
 static void set_multicast_list(struct net_device *dev);
 static int set_card_type(dev_link_t *link, const void *s);
@@ -612,29 +597,6 @@ mii_wr(ioaddr_t ioaddr, u_char phyaddr, u_char phyreg, unsigned data, int len)
     mii_idle(ioaddr);
 }
 
-#ifdef PCMCIA_DEBUG
-static void
-mii_dump(struct net_device *dev)
-{
-    ioaddr_t ioaddr = dev->base_addr;
-    int i;
-
-    /* Note that registers 14, 1d,1e and 1f are reserved and should
-     * not be read according to the DP83840A specs.
-     */
-    printk(KERN_DEBUG "%s: MII register dump:\n", dev->name);
-    for (i=0; i < 32; i++) {
-	if (!(i % 8)) {
-	    if (i)
-		printk("\n");
-	    printk(KERN_DEBUG "%s:", dev->name);
-	}
-	printk(" %04x", mii_rd(ioaddr, 0, i));
-    }
-    printk("\n");
-}
-#endif
-
 /*============= Main bulk of functions	=========================*/
 
 /****************
@@ -912,26 +874,26 @@ xirc2ps_config(dev_link_t * link)
     switch(parse.manfid.manf) {
       case MANFID_XIRCOM:
 	local->manf_str = "Xircom";
-	DEBUG(0, "found xircom card\n");
 	break;
       case MANFID_ACCTON:
 	local->manf_str = "Accton";
-	DEBUG(0, "found Accton card\n");
 	break;
       case MANFID_COMPAQ:
       case MANFID_COMPAQ2:
 	local->manf_str = "Compaq";
-	DEBUG(0, "found Compaq card\n");
 	break;
       case MANFID_INTEL:
 	local->manf_str = "Intel";
-	DEBUG(0, "found Intel card\n");
+	break;
+      case MANFID_TOSHIBA:
+	local->manf_str = "Toshiba";
 	break;
       default:
 	printk(KNOT_XIRC "Unknown Card Manufacturer ID: 0x%04x\n",
 	       (unsigned)parse.manfid.manf);
 	goto failure;
     }
+    DEBUG(0, "found %s card\n", local->manf_str);
 
     if (!set_card_type(link, buf)) {
 	printk(KNOT_XIRC "this card is not supported\n");
@@ -1621,7 +1583,7 @@ do_start_xmit(struct sk_buff *skb, struct net_device *dev)
     return 0;
 }
 
-static struct enet_statistics *
+static struct net_device_stats *
 do_get_stats(struct net_device *dev)
 {
     local_info_t *lp = dev->priv;
@@ -1721,12 +1683,6 @@ do_config(struct net_device *dev, struct ifmap *map)
 	       dev->name, if_names[dev->if_port]);
 	do_reset(dev,1);  /* not the fine way :-) */
     }
-  #ifdef PCMCIA_DEBUG
-    else if (local->mohawk) {
-	/* kludge to print the mii regsiters */
-	mii_dump(dev);
-    }
-  #endif
     return 0;
 }
 
@@ -1919,6 +1875,8 @@ do_reset(struct net_device *dev, int full)
 		PutByte(XIRCREG42_SWC1, 0x80);
 	    busy_loop(HZ/25);	/* wait 40 msec to let it complete */
 	}
+	if (full_duplex)
+	    PutByte(XIRCREG1_ECR, GetByte(XIRCREG1_ECR | FullDuplex));
     } else {  /* No MII */
 	SelectPage(0);
 	value = GetByte(XIRCREG_ESR);	 /* read the ESR */
@@ -1973,12 +1931,6 @@ init_mii(struct net_device *dev)
     unsigned control, status, linkpartner;
     int i;
 
-  #ifdef PCMCIA_DEBUG
-    if (pc_debug>1) {
-	mii_dump(dev);
-    }
-  #endif
-
     status = mii_rd(ioaddr,  0, 1);
     if ((status & 0xff00) != 0x7800)
 	return 0; /* No MII */
@@ -2035,11 +1987,6 @@ init_mii(struct net_device *dev)
 	}
     }
 
-  #ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	mii_dump(dev);
-  #endif
-
     return 1;
 }
 
@@ -2094,8 +2041,6 @@ init_xirc2ps_cs(void)
     servinfo_t serv;
 
     printk(KERN_INFO "%s\n", version);
-    if (card_type)
-	printk(KINF_XIRC "option card_type is obsolete\n");
     if (lockup_hack)
 	printk(KINF_XIRC "lockup hack is enabled\n");
     CardServices(GetCardServicesInfo, &serv);
