@@ -2,7 +2,7 @@
 
     Register dump for the Databook TCIC-2 controller family
 
-    dump_tcic.c 1.20 1999/10/25 19:57:59
+    dump_tcic.c 1.21 2000/06/12 21:34:19
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,7 +15,7 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    <dahinds@users.sourceforge.net>.  Portions created by David A. Hinds
     are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
 
     Alternatively, the contents of this file may be used under the
