@@ -2,7 +2,7 @@
   
     Cardbus device configuration
     
-    cardbus.c 1.74 2000/05/20 03:16:31
+    cardbus.c 1.78 2000/10/26 20:10:55
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,7 +15,7 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    <dahinds@users.sourceforge.net>.  Portions created by David A. Hinds
     are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
 
     Alternatively, the contents of this file may be used under the
@@ -194,7 +194,7 @@ static u_int xlate_rom_addr(u_char *b, u_int addr)
     
 =====================================================================*/
 
-int cb_setup_cis_mem(socket_info_t *s, int space)
+static int cb_setup_cis_mem(socket_info_t *s, int space)
 {
     cb_bridge_map *m = &s->cb_cis_map;
     cb_config_t *c = s->cb_config;
@@ -314,9 +314,11 @@ int cb_alloc(socket_info_t *s)
     u_short vend, v, dev;
     u_char hdr, fn, bus = s->cap.cardbus;
     cb_config_t *c;
-
     struct pci_dev tmp;
     int i;
+
+    if (s->cb_config)
+	return CS_SUCCESS;
     tmp.bus = s->cap.cb_bus; tmp.devfn = 0;
 #ifdef NEWER_LINUX_PCI
     list_add(&tmp.global_list, &pci_devices);
@@ -531,7 +533,7 @@ int cb_config(socket_info_t *s)
     for (i = 0; i < fn; i++) {
 	pci_readb(&c[i].dev, PCI_INTERRUPT_PIN, &j);
 	if (j == 0) continue;
-	if ((irq == 0) && (s->cap.irq_mask & (1 << s->cap.pci_irq)))
+	if ((irq == 0) && s->cap.pci_irq)
 	    irq = s->cap.pci_irq;
 #ifdef CONFIG_ISA
 	if (irq == 0) {
