@@ -2,7 +2,7 @@
 
     PCMCIA Card Services -- core services
 
-    cs.c 1.259 2000/05/10 19:26:32
+    cs.c 1.272 2000/11/07 19:09:01
     
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,7 +15,7 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    <dahinds@users.sourceforge.net>.  Portions created by David A. Hinds
     are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
 
     Alternatively, the contents of this file may be used under the
@@ -50,6 +50,7 @@
 #include <linux/delay.h>
 #include <linux/proc_fs.h>
 #include <linux/pm.h>
+#include <linux/pci.h>
 #include <asm/system.h>
 #include <asm/irq.h>
 #endif
@@ -70,7 +71,7 @@
 int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
-"cs.c 1.259 2000/05/10 19:26:32 (David Hinds)";
+"cs.c 1.272 2000/11/07 19:09:01 (David Hinds)";
 #endif
 
 #ifdef CONFIG_PCI
@@ -111,7 +112,7 @@ static const char *kernel = "kernel build: " UTS_RELEASE " " UTS_VERSION;
 #endif
 static const char *options = "options: " OPTIONS;
 
-MODULE_AUTHOR("David Hinds <dhinds@pcmcia.sourceforge.org>");
+MODULE_AUTHOR("David Hinds <dahinds@users.sourceforge.net>");
 MODULE_DESCRIPTION("Linux PCMCIA Card Services " CS_RELEASE
 		   "\n  options:" OPTIONS);
 
@@ -128,7 +129,7 @@ INT_MODULE_PARM(vcc_settle,	HZ*4/10);	/* ticks */
 INT_MODULE_PARM(reset_time,	10);		/* usecs */
 INT_MODULE_PARM(unreset_delay,	HZ/10);		/* ticks */
 INT_MODULE_PARM(unreset_check,	HZ/10);		/* ticks */
-INT_MODULE_PARM(unreset_limit,	30);		/* unreset_check's */
+INT_MODULE_PARM(unreset_limit,	50);		/* unreset_check's */
 
 /* Access speed for attribute memory windows */
 INT_MODULE_PARM(cis_speed,	300);		/* ns */
@@ -481,8 +482,7 @@ static void setup_socket(u_long i)
 	    printk(KERN_NOTICE "cs: socket %ld voltage interrogation"
 		   " timed out\n", i);
 	} else {
-	    s->setup.expires = jiffies + HZ/10;
-	    add_timer(&s->setup);
+	    mod_timer(&s->setup, jiffies + HZ/10);
 	}
     } else if (val & SS_DETECT) {
 	DEBUG(1, "cs: setup_socket(%ld): applying power\n", i);
@@ -505,8 +505,7 @@ static void setup_socket(u_long i)
 	}
 	s->ss_entry(s->sock, SS_SetSocket, &s->socket);
 	s->setup.function = &reset_socket;
-	s->setup.expires = jiffies + vcc_settle;
-	add_timer(&s->setup);
+	mod_timer(&s->setup, jiffies + vcc_settle);
     } else
 	DEBUG(0, "cs: setup_socket(%ld): no card!\n", i);
 } /* setup_socket */
@@ -531,9 +530,8 @@ static void reset_socket(u_long i)
     s->socket.flags &= ~SS_RESET;
     s->ss_entry(s->sock, SS_SetSocket, &s->socket);
     s->setup_timeout = 0;
-    s->setup.expires = jiffies + unreset_delay;
     s->setup.function = &unreset_socket;
-    add_timer(&s->setup);
+    mod_timer(&s->setup, jiffies + unreset_delay);
 } /* reset_socket */
 
 #define EVENT_MASK \
@@ -562,10 +560,12 @@ static void unreset_socket(u_long i)
 	    s->state &= ~SOCKET_SETUP_PENDING;
 	} else {
 	    send_event(s, CS_EVENT_CARD_RESET, CS_EVENT_PRI_LOW);
-	    s->reset_handle->event_callback_args.info = NULL;
-	    EVENT(s->reset_handle, CS_EVENT_RESET_COMPLETE,
-		  CS_EVENT_PRI_LOW);
-	    s->state &= ~EVENT_MASK;
+	    if (s->reset_handle) {
+		s->reset_handle->event_callback_args.info = NULL;
+		EVENT(s->reset_handle, CS_EVENT_RESET_COMPLETE,
+		      CS_EVENT_PRI_LOW);
+		s->state &= ~EVENT_MASK;
+	    }
 	}
     } else {
 	DEBUG(2, "cs: socket %ld not ready yet\n", i);
@@ -574,8 +574,7 @@ static void unreset_socket(u_long i)
 		   " reset\n", i);
 	    s->state &= ~EVENT_MASK;
 	} else {
-	    s->setup.expires = jiffies + unreset_check;
-	    add_timer(&s->setup);
+	    mod_timer(&s->setup, jiffies + unreset_check);
 	}
     }
 } /* unreset_socket */
@@ -623,8 +622,7 @@ static void do_shutdown(socket_info_t *s)
 	del_timer(&s->setup);
 	s->state &= ~EVENT_MASK;
     }
-    s->shutdown.expires = jiffies + shutdown_delay;
-    add_timer(&s->shutdown);
+    mod_timer(&s->shutdown, jiffies + shutdown_delay);
     s->state &= ~SOCKET_PRESENT;
 }
 
@@ -756,6 +754,13 @@ static int alloc_io_space(socket_info_t *s, u_int attr, ioaddr_t *base,
 	      *base, align);
 	align = 0;
     }
+    /* Check for an already-allocated window that must conflict with
+       what was asked for.  It is a hack because it does not catch all
+       potential conflicts, just the most obvious ones. */
+    for (i = 0; i < MAX_IO_WIN; i++)
+	if ((s->io[i].NumPorts != 0) &&
+	    ((s->io[i].BasePort & (align-1)) == *base))
+	    return 1;
     for (i = 0; i < MAX_IO_WIN; i++) {
 	if (s->io[i].NumPorts == 0) {
 	    if (find_io_region(base, num, align, name) == 0) {
@@ -960,8 +965,10 @@ static int deregister_client(client_handle_t handle)
 	client = &s->clients;
 	while ((*client) && ((*client) != handle))
 	    client = &(*client)->next;
-	if (*client == NULL)
+	if (*client == NULL) {
+	    spin_unlock_irqrestore(&s->lock, flags);
 	    return CS_BAD_HANDLE;
+	}
 	*client = handle->next;
 	handle->client_magic = 0;
 	kfree(handle);
@@ -1006,9 +1013,11 @@ static int get_configuration_info(client_handle_t handle,
 	config->Vcc = s->socket.Vcc;
 	config->Vpp1 = config->Vpp2 = s->socket.Vpp;
 	config->Option = s->cap.cardbus;
+	config->IntType = INT_CARDBUS;
+	/* This is a nasty hack */
+	pcibios_read_config_dword(s->cap.cardbus, 0, 0, &config->ConfigBase);
 	if (s->cb_config) {
 	    config->Attributes = CONF_VALID_CLIENT;
-	    config->IntType = INT_CARDBUS;
 	    config->AssignedIRQ = s->irq.AssignedIRQ;
 	    if (config->AssignedIRQ)
 		config->Attributes |= CONF_ENABLE_IRQ;
@@ -1642,17 +1651,19 @@ static int request_configuration(client_handle_t handle,
 	write_cis_mem(s, 1, (base + CISREG_SCR)>>1, 1, &c->Copy);
     }
     if (req->Present & PRESENT_OPTION) {
-	if (s->functions == 1)
+	if (s->functions == 1) {
 	    c->Option = req->ConfigIndex & COR_CONFIG_MASK;
-	else {
+	} else {
 	    c->Option = req->ConfigIndex & COR_MFC_CONFIG_MASK;
-	    c->Option |= COR_FUNC_ENA|COR_ADDR_DECODE|COR_IREQ_ENA;
+	    c->Option |= COR_FUNC_ENA|COR_IREQ_ENA;
+	    if (req->Present & PRESENT_IOBASE_0)
+		c->Option |= COR_ADDR_DECODE;
 	}
 	if (c->state & CONFIG_IRQ_REQ)
 	    if (!(c->irq.Attributes & IRQ_FORCED_PULSE))
 		c->Option |= COR_LEVEL_REQ;
 	write_cis_mem(s, 1, (base + CISREG_COR)>>1, 1, &c->Option);
-	udelay(40*1000);
+	mdelay(40);
     }
     if (req->Present & PRESENT_STATUS) {
 	c->Status = req->Status;
@@ -1667,14 +1678,14 @@ static int request_configuration(client_handle_t handle,
 	write_cis_mem(s, 1, (base + CISREG_ESR)>>1, 1, &c->ExtStatus);
     }
     if (req->Present & PRESENT_IOBASE_0) {
-	i = c->io.BasePort1 & 0xff;
-	write_cis_mem(s, 1, (base + CISREG_IOBASE_0)>>1, 1, &i);
-	i = (c->io.BasePort1 >> 8) & 0xff;
-	write_cis_mem(s, 1, (base + CISREG_IOBASE_1)>>1, 1, &i);
+	u_char b = c->io.BasePort1 & 0xff;
+	write_cis_mem(s, 1, (base + CISREG_IOBASE_0)>>1, 1, &b);
+	b = (c->io.BasePort1 >> 8) & 0xff;
+	write_cis_mem(s, 1, (base + CISREG_IOBASE_1)>>1, 1, &b);
     }
     if (req->Present & PRESENT_IOSIZE) {
-	i = c->io.NumPorts1 + c->io.NumPorts2 - 1;
-	write_cis_mem(s, 1, (base + CISREG_IOSIZE)>>1, 1, &i);
+	u_char b = c->io.NumPorts1 + c->io.NumPorts2 - 1;
+	write_cis_mem(s, 1, (base + CISREG_IOSIZE)>>1, 1, &b);
     }
     
     /* Configure I/O windows */
@@ -1781,8 +1792,7 @@ static int cs_request_irq(client_handle_t handle, irq_req_t *req)
 {
     socket_info_t *s;
     config_t *c;
-    int try, ret = 0, irq = 0;
-    u_int mask;
+    int ret = 0, irq = 0;
     
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
@@ -1795,23 +1805,23 @@ static int cs_request_irq(client_handle_t handle, irq_req_t *req)
     if (c->state & CONFIG_IRQ_REQ)
 	return CS_IN_USE;
     
-    /* Short cut: if the interrupt is PCI, there are no options */
-    if (s->cap.irq_mask == (1 << s->cap.pci_irq))
+    /* Short cut: if there are no ISA interrupts, then it is PCI */
+    if (!s->cap.irq_mask) {
 	irq = s->cap.pci_irq;
+	ret = (irq) ? 0 : CS_IN_USE;
 #ifdef CONFIG_ISA
-    else if (s->irq.AssignedIRQ != 0) {
+    } else if (s->irq.AssignedIRQ != 0) {
 	/* If the interrupt is already assigned, it must match */
 	irq = s->irq.AssignedIRQ;
 	if (req->IRQInfo1 & IRQ_INFO2_VALID) {
-	    mask = req->IRQInfo2 & s->cap.irq_mask;
+	    u_int mask = req->IRQInfo2 & s->cap.irq_mask;
 	    ret = ((mask >> irq) & 1) ? 0 : CS_BAD_ARGS;
 	} else
 	    ret = ((req->IRQInfo1&IRQ_MASK) == irq) ? 0 : CS_BAD_ARGS;
     } else {
 	ret = CS_IN_USE;
 	if (req->IRQInfo1 & IRQ_INFO2_VALID) {
-	    mask = req->IRQInfo2 & s->cap.irq_mask;
-	    mask &= ~(1 << s->cap.pci_irq);
+	    u_int try, mask = req->IRQInfo2 & s->cap.irq_mask;
 	    for (try = 0; try < 2; try++) {
 		for (irq = 0; irq < 32; irq++)
 		    if ((mask >> irq) & 1) {
@@ -1824,8 +1834,8 @@ static int cs_request_irq(client_handle_t handle, irq_req_t *req)
 	    irq = req->IRQInfo1 & IRQ_MASK;
 	    ret = try_irq(req->Attributes, irq, 1);
 	}
-    }
 #endif
+    }
     if (ret != 0) return ret;
 
     if (req->Attributes & IRQ_HANDLE_PRESENT) {
