@@ -3,7 +3,7 @@
     Device driver for Intel 82365 and compatible PC Card controllers,
     and Yenta-compatible PCI-to-CardBus controllers.
 
-    i82365.c 1.317 2000/06/09 22:14:25
+    i82365.c 1.326 2000/10/02 20:27:49
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -16,7 +16,7 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    <dahinds@users.sourceforge.net>.  Portions created by David A. Hinds
     are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
 
     Alternatively, the contents of this file may be used under the
@@ -81,7 +81,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static const char *version =
-"i82365.c 1.317 2000/06/09 22:14:25 (David Hinds)";
+"i82365.c 1.326 2000/10/02 20:27:49 (David Hinds)";
 #else
 #define DEBUG(n, args...) do { } while (0)
 #endif
@@ -106,7 +106,7 @@ typedef void irq_ret_t;
 #define _free_irq(i, h) free_irq(i, socket)
 #endif
 
-MODULE_AUTHOR("David Hinds <dhinds@pcmcia.sourceforge.org>");
+MODULE_AUTHOR("David Hinds <dahinds@users.sourceforge.net>");
 MODULE_DESCRIPTION("Intel ExCA/Yenta PCMCIA socket driver");
 
 /*====================================================================*/
@@ -118,6 +118,7 @@ MODULE_DESCRIPTION("Intel ExCA/Yenta PCMCIA socket driver");
 /* General options */
 INT_MODULE_PARM(poll_interval, 0);	/* in ticks, 0 means never */
 INT_MODULE_PARM(cycle_time, 120);	/* in ns, 120 ns = 8.33 MHz */
+INT_MODULE_PARM(do_scan, 1);		/* Probe free interrupts? */
 
 /* Cirrus options */
 INT_MODULE_PARM(has_dma, -1);
@@ -133,15 +134,13 @@ INT_MODULE_PARM(recov_time, -1);
 INT_MODULE_PARM(i365_base, 0x3e0);	/* IO address for probes */
 INT_MODULE_PARM(extra_sockets, 0);	/* Probe at i365_base+2? */
 INT_MODULE_PARM(ignore, -1);		/* Ignore this socket # */
-INT_MODULE_PARM(do_scan, 1);		/* Probe free interrupts? */
 INT_MODULE_PARM(cs_irq, 0);		/* card status irq */
 INT_MODULE_PARM(irq_mask, 0xffff);	/* bit map of irq's to use */
 static int irq_list[16] = { -1 };
 MODULE_PARM(irq_list, "1-16i");
-/* Vadem options */
-INT_MODULE_PARM(async_clock, -1);
+INT_MODULE_PARM(async_clock, -1);	/* Vadem specific */
 INT_MODULE_PARM(cable_mode, -1);
-INT_MODULE_PARM(wakeup, 0);
+INT_MODULE_PARM(wakeup, 0);		/* Cirrus specific */
 #endif
 
 #ifdef CONFIG_PCI
@@ -484,7 +483,10 @@ static u_int __init cirrus_set_opts(socket_info_t *s, char *buf)
 	}
 #endif
     }
-    if (s->type != IS_VT83C469) {
+#ifdef CONFIG_ISA
+    if (s->type != IS_VT83C469)
+#endif
+    {
 	if (setup_time >= 0)
 	    p->timer[0] = p->timer[3] = setup_time;
 	if (cmd_time > 0) {
@@ -575,6 +577,7 @@ static void __init ti113x_get_state(socket_info_t *s)
     pci_readb(s, TI113X_CARD_CONTROL, &p->cardctl);
     pci_readb(s, TI113X_DEVICE_CONTROL, &p->devctl);
     pci_readb(s, TI1250_DIAGNOSTIC, &p->diag);
+    pci_readl(s, TI12XX_IRQMUX, &p->irqmux);
 }
 
 static void ti113x_set_state(socket_info_t *s)
@@ -585,6 +588,7 @@ static void ti113x_set_state(socket_info_t *s)
     pci_writeb(s, TI113X_DEVICE_CONTROL, p->devctl);
     pci_writeb(s, TI1250_MULTIMEDIA_CTL, 0);
     pci_writeb(s, TI1250_DIAGNOSTIC, p->diag);
+    pci_writel(s, TI12XX_IRQMUX, p->irqmux);
     i365_set_pair(s, TI113X_IO_OFFSET(0), 0);
     i365_set_pair(s, TI113X_IO_OFFSET(1), 0);
 }
@@ -623,6 +627,7 @@ static u_int __init ti113x_set_opts(socket_info_t *s, char *buf)
     switch (irq_mode) {
     case 0:
 	p->devctl &= ~TI113X_DCR_IMODE_MASK;
+	p->irqmux |= 0x02; /* minimal routing for INTA */
 	break;
     case 1:
 	p->devctl &= ~TI113X_DCR_IMODE_MASK;
@@ -851,7 +856,7 @@ static u_int __init o2micro_set_opts(socket_info_t *s, char *buf)
 	case O2_MODE_D_IRQ_PCIWAY:
 	    strcat(buf, " [pci/way]"); break;
 	case O2_MODE_D_IRQ_PCI:
-	    strcat(buf, " [pci only]"); break;
+	    strcat(buf, " [pci only]"); mask = 0; break;
 	}
     }
     if (s->flags & IS_CARDBUS) {
@@ -891,18 +896,6 @@ static void topic_set_state(socket_info_t *s)
     pci_writel(s, TOPIC_REGISTER_CONTROL, p->rcr);
 }
 
-static int topic_set_irq_mode(socket_info_t *s, int pcsc, int pint)
-{
-    if (s->type >= IS_TOPIC97) {
-	topic_state_t *p = &s->state.topic;
-	flip(p->ccr, TOPIC97_ICR_IRQSEL, pcsc);
-	return 0;
-    } else {
-	/* no ISA card status change irq */
-	return !pcsc;
-    }
-}
-
 static u_int __init topic_set_opts(socket_info_t *s, char *buf)
 {
     topic_state_t *p = &s->state.topic;
@@ -968,8 +961,6 @@ static int cb_set_irq_mode(socket_info_t *s, int pcsc, int pint)
 	return cirrus_set_irq_mode(s, pcsc, pint);
     else if (s->flags & IS_TI)
 	return ti113x_set_irq_mode(s, pcsc, pint);
-    else if (s->flags & IS_TOPIC)
-	return topic_set_irq_mode(s, pcsc, pint);
     /* By default, assume that we can't do ISA status irqs */
     return (!pcsc);
 }
@@ -1082,11 +1073,13 @@ static void set_bridge_state(socket_info_t *s)
     if (s->flags & IS_CARDBUS)
 	cb_set_state(s);
 #endif
-    if (s->flags & IS_CIRRUS)
+    if (s->flags & IS_CIRRUS) {
 	cirrus_set_state(s);
-    else {
+    } else {
 	i365_set(s, I365_GBLCTL, 0x00);
 	i365_set(s, I365_GENCTL, 0x00);
+	/* Trouble: changes timing of memory operations */
+	/* i365_set(s, I365_ADDRWIN, I365_ADDR_MEMCS16); */
     }
     i365_bflip(s, I365_INTCTL, I365_INTR_ENA, s->intr);
 #ifdef CONFIG_ISA
@@ -1175,21 +1168,6 @@ static irq_ret_t irq_count IRQ(int irq, void *dev, struct pt_regs *regs)
     return (irq_ret_t)1;
 }
 
-#ifdef __LINUX__
-static int _check_irq(int irq, int flags)
-{
-#ifdef CONFIG_PNP_BIOS
-    extern int check_pnp_irq(int);
-    if ((flags != SA_SHIRQ) && check_pnp_irq(irq))
-	return -1;
-#endif
-    if (request_irq(irq, irq_count, flags, "x", irq_count) != 0)
-	return -1;
-    free_irq(irq, irq_count);
-    return 0;
-}
-#endif
-
 static u_int __init test_irq(socket_info_t *s, int irq, int pci)
 {
     u_char csc = (pci) ? 0 : irq;
@@ -1223,7 +1201,7 @@ static u_int __init test_irq(socket_info_t *s, int irq, int pci)
 	cb_writel(s, CB_SOCKET_EVENT, -1);
 	cb_writel(s, CB_SOCKET_MASK, CB_SM_CSTSCHG);
 	cb_writel(s, CB_SOCKET_FORCE, CB_SE_CSTSCHG);
-	udelay(1000);
+	mdelay(1);
 	cb_writel(s, CB_SOCKET_EVENT, -1);
 	cb_writel(s, CB_SOCKET_MASK, 0);
     } else
@@ -1231,7 +1209,7 @@ static u_int __init test_irq(socket_info_t *s, int irq, int pci)
     {
 	i365_set(s, I365_CSCINT, I365_CSC_DETECT | (csc << 4));
 	i365_bset(s, I365_GENCTL, I365_CTL_SW_IRQ);
-	udelay(1000);
+	mdelay(1);
     }
 
     _free_irq(irq, irq_count);
@@ -1244,6 +1222,21 @@ static u_int __init test_irq(socket_info_t *s, int irq, int pci)
 }
 
 #ifdef CONFIG_ISA
+#ifdef __LINUX__
+static int _check_irq(int irq, int flags)
+{
+#ifdef CONFIG_PNP_BIOS
+    extern int check_pnp_irq(int);
+    if ((flags != SA_SHIRQ) && check_pnp_irq(irq))
+	return -1;
+#endif
+    if (request_irq(irq, irq_count, flags, "x", irq_count) != 0)
+	return -1;
+    free_irq(irq, irq_count);
+    return 0;
+}
+#endif
+
 static u_int __init isa_scan(socket_info_t *s, u_int mask0)
 {
     u_int mask1 = 0;
@@ -1291,7 +1284,7 @@ static u_int __init isa_scan(socket_info_t *s, u_int mask0)
 static int __init pci_scan(socket_info_t *s)
 {
     int ret;
-    if ((s->flags & IS_RICOH) || !(s->flags & IS_CARDBUS)) {
+    if ((s->flags & IS_RICOH) || !(s->flags & IS_CARDBUS) || !do_scan) {
 	/* for PCI-to-PCMCIA bridges, just check for wedged irq */
 	irq_sock = s; irq_hits = 0;
 	if (_request_irq(s->cap.pci_irq, irq_count, 0, "scan"))
@@ -1513,14 +1506,11 @@ static void __init add_pcic(int ns, int type)
 	s[i].cap.irq_mask = mask;
 	if (!use_pci)
 	    s[i].cap.pci_irq = 0;
-	if (use_pci && pci_int)
-	    s[i].cap.irq_mask |= (1 << s[i].cap.pci_irq);
 	s[i].cs_irq = isa_irq;
 #ifdef CONFIG_PCI
 	if (s[i].flags & IS_CARDBUS) {
 	    s[i].cap.features |= SS_CAP_CARDBUS;
-	    cb_set_irq_mode(s+i, pci_csc && s[i].cap.pci_irq,
-			    pci_int && s[i].cap.pci_irq);
+	    cb_set_irq_mode(s+i, pci_csc && s[i].cap.pci_irq, 0);
 	}
 #endif
     }
@@ -1581,11 +1571,12 @@ static void __init add_pci_bridge(int type, u_short v, u_short d)
 
 static int check_cb_mapping(socket_info_t *s)
 {
+    u_int state = cb_readl(s, CB_SOCKET_STATE) >> 16;
     /* A few sanity checks to validate the bridge mapping */
     if ((cb_readb(s, 0x800+I365_IDENT) & 0x70) ||
 	(cb_readb(s, 0x800+I365_CSC) && cb_readb(s, 0x800+I365_CSC) &&
 	 cb_readb(s, 0x800+I365_CSC)) || cb_readl(s, CB_SOCKET_FORCE) ||
-	((cb_readl(s, CB_SOCKET_STATE) >> 16) != 0x3000))
+	((state & ~0x3000) || !(state & 0x3000)))
 	return 1;
     return 0;
 }
@@ -1780,9 +1771,9 @@ static irq_ret_t pcic_interrupt IRQ(int irq, void *dev,
 		continue;
 	    }
 	    events = (csc & I365_CSC_DETECT) ? SS_DETECT : 0;
-	    if (i365_get(s, I365_INTCTL) & I365_PC_IOCARD)
+	    if (i365_get(s, I365_INTCTL) & I365_PC_IOCARD) {
 		events |= (csc & I365_CSC_STSCHG) ? SS_STSCHG : 0;
-	    else {
+	    } else {
 		events |= (csc & I365_CSC_BVD1) ? SS_BATDEAD : 0;
 		events |= (csc & I365_CSC_BVD2) ? SS_BATWARN : 0;
 		events |= (csc & I365_CSC_READY) ? SS_READY : 0;
@@ -1846,9 +1837,9 @@ static int i365_get_status(socket_info_t *s, u_int *value)
     status = i365_get(s, I365_STATUS);
     *value = ((status & I365_CS_DETECT) == I365_CS_DETECT)
 	? SS_DETECT : 0;
-    if (i365_get(s, I365_INTCTL) & I365_PC_IOCARD)
+    if (i365_get(s, I365_INTCTL) & I365_PC_IOCARD) {
 	*value |= (status & I365_CS_STSCHG) ? 0 : SS_STSCHG;
-    else {
+    } else {
 	*value |= (status & I365_CS_BVD1) ? 0 : SS_BATDEAD;
 	*value |= (status & I365_CS_BVD2) ? 0 : SS_BATWARN;
     }
@@ -1954,9 +1945,9 @@ static int i365_get_socket(socket_info_t *s, socket_state_t *state)
     /* Card status change mask */
     reg = i365_get(s, I365_CSCINT);
     state->csc_mask = (reg & I365_CSC_DETECT) ? SS_DETECT : 0;
-    if (state->flags & SS_IOCARD)
+    if (state->flags & SS_IOCARD) {
 	state->csc_mask |= (reg & I365_CSC_STSCHG) ? SS_STSCHG : 0;
-    else {
+    } else {
 	state->csc_mask |= (reg & I365_CSC_BVD1) ? SS_BATDEAD : 0;
 	state->csc_mask |= (reg & I365_CSC_BVD2) ? SS_BATWARN : 0;
 	state->csc_mask |= (reg & I365_CSC_READY) ? SS_READY : 0;
