@@ -1,5 +1,5 @@
 /*
- * ti113x.h 1.22 2000/06/09 21:26:14
+ * ti113x.h 1.26 2000/11/07 21:15:41
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.1 (the "License"); you may not use this file except in
@@ -12,7 +12,7 @@
  * limitations under the License. 
  *
  * The initial developer of the original code is David A. Hinds
- * <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+ * <dahinds@users.sourceforge.net>.  Portions created by David A. Hinds
  * are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
  *
  * Alternatively, the contents of this file may be used under the
@@ -76,11 +76,14 @@
 #ifndef PCI_DEVICE_ID_TI_1420
 #define PCI_DEVICE_ID_TI_1420		0xac51
 #endif
+#ifndef PCI_DEVICE_ID_TI_4451
+#define PCI_DEVICE_ID_TI_4451		0xac42
+#endif
 
 /* Register definitions for TI 113X PCI-to-CardBus bridges */
 
 /* System Control Register */
-#define TI113X_SYSTEM_CONTROL		0x0080	/* 32 bit */
+#define TI113X_SYSTEM_CONTROL		0x80	/* 32 bit */
 #define  TI113X_SCR_SMIROUTE		0x04000000
 #define  TI113X_SCR_SMISTATUS		0x02000000
 #define  TI113X_SCR_SMIENB		0x01000000
@@ -111,21 +114,21 @@
 #define  TI122X_SCR_RIMUX		0x00000001
 
 /* Multimedia Control Register */
-#define TI1250_MULTIMEDIA_CTL		0x0084	/* 8 bit */
+#define TI1250_MULTIMEDIA_CTL		0x84	/* 8 bit */
 #define  TI1250_MMC_ZVOUTEN		0x80
 #define  TI1250_MMC_PORTSEL		0x40
 #define  TI1250_MMC_ZVEN1		0x02
 #define  TI1250_MMC_ZVEN0		0x01
 
-#define TI1250_GENERAL_STATUS		0x0085	/* 8 bit */
-#define TI1250_GPIO0_CONTROL		0x0088	/* 8 bit */
-#define TI1250_GPIO1_CONTROL		0x0089	/* 8 bit */
-#define TI1250_GPIO2_CONTROL		0x008a	/* 8 bit */
-#define TI1250_GPIO3_CONTROL		0x008b	/* 8 bit */
-#define TI122X_IRQMUX			0x008c	/* 32 bit */
+#define TI1250_GENERAL_STATUS		0x85	/* 8 bit */
+#define TI1250_GPIO0_CONTROL		0x88	/* 8 bit */
+#define TI1250_GPIO1_CONTROL		0x89	/* 8 bit */
+#define TI1250_GPIO2_CONTROL		0x8a	/* 8 bit */
+#define TI1250_GPIO3_CONTROL		0x8b	/* 8 bit */
+#define TI12XX_IRQMUX			0x8c	/* 32 bit */
 
 /* Retry Status Register */
-#define TI113X_RETRY_STATUS		0x0090	/* 8 bit */
+#define TI113X_RETRY_STATUS		0x90	/* 8 bit */
 #define  TI113X_RSR_PCIRETRY		0x80
 #define  TI113X_RSR_CBRETRY		0x40
 #define  TI113X_RSR_TEXP_CBB		0x20
@@ -136,7 +139,7 @@
 #define  TI113X_RSR_MEXP_PCI		0x01
 
 /* Card Control Register */
-#define TI113X_CARD_CONTROL		0x0091	/* 8 bit */
+#define TI113X_CARD_CONTROL		0x91	/* 8 bit */
 #define  TI113X_CCR_RIENB		0x80
 #define  TI113X_CCR_ZVENABLE		0x40
 #define  TI113X_CCR_PCI_IRQ_ENA		0x20
@@ -149,7 +152,7 @@
 #define  TI122X_CCR_AUD2MUX		0x04
 
 /* Device Control Register */
-#define TI113X_DEVICE_CONTROL		0x0092	/* 8 bit */
+#define TI113X_DEVICE_CONTROL		0x92	/* 8 bit */
 #define  TI113X_DCR_5V_FORCE		0x40
 #define  TI113X_DCR_3V_FORCE		0x20
 #define  TI113X_DCR_IMODE_MASK		0x06
@@ -160,38 +163,39 @@
 #define  TI12XX_DCR_IMODE_ALL_SERIAL	0x06
 
 /* Buffer Control Register */
-#define TI113X_BUFFER_CONTROL		0x0093	/* 8 bit */
+#define TI113X_BUFFER_CONTROL		0x93	/* 8 bit */
 #define  TI113X_BCR_CB_READ_DEPTH	0x08
 #define  TI113X_BCR_CB_WRITE_DEPTH	0x04
 #define  TI113X_BCR_PCI_READ_DEPTH	0x02
 #define  TI113X_BCR_PCI_WRITE_DEPTH	0x01
 
 /* Diagnostic Register */
-#define TI1250_DIAGNOSTIC		0x0093	/* 8 bit */
+#define TI1250_DIAGNOSTIC		0x93	/* 8 bit */
 #define  TI1250_DIAG_TRUE_VALUE		0x80
 #define  TI1250_DIAG_PCI_IREQ		0x40
 #define  TI1250_DIAG_PCI_CSC		0x20
 #define  TI1250_DIAG_ASYNC_CSC		0x01
 
 /* DMA Registers */
-#define TI113X_DMA_0			0x0094	/* 32 bit */
-#define TI113X_DMA_1			0x0098	/* 32 bit */
+#define TI113X_DMA_0			0x94	/* 32 bit */
+#define TI113X_DMA_1			0x98	/* 32 bit */
 
 /* ExCA IO offset registers */
 #define TI113X_IO_OFFSET(map)		(0x36+((map)<<1))
 
 /* Data structure for tracking vendor-specific state */
 typedef struct ti113x_state_t {
-    u_int		sysctl;		/* TI113X_SYSTEM_CONTROL */
-    u_char		cardctl;	/* TI113X_CARD_CONTROL */
-    u_char		devctl;		/* TI113X_DEVICE_CONTROL */
-    u_char		diag;		/* TI1250_DIAGNOSTIC */
+    u32			sysctl;		/* TI113X_SYSTEM_CONTROL */
+    u8			cardctl;	/* TI113X_CARD_CONTROL */
+    u8			devctl;		/* TI113X_DEVICE_CONTROL */
+    u8			diag;		/* TI1250_DIAGNOSTIC */
+    u32			irqmux;		/* TI12XX_IRQMUX */
 } ti113x_state_t;
 
 #define TI_PCIC_ID \
     IS_TI1130, IS_TI1131, IS_TI1250A, IS_TI1220, IS_TI1221,	\
     IS_TI1210, IS_TI1251A, IS_TI1251B, IS_TI1450, IS_TI1225,	\
-    IS_TI1211, IS_TI1420, IS_TI1031, IS_TI1410
+    IS_TI1211, IS_TI1420, IS_TI1031, IS_TI1410, IS_TI4451
 
 #define TI_PCIC_INFO \
     { "TI 1130",  IS_TI|IS_CARDBUS, ID(TI, 1130) }, \
@@ -207,6 +211,7 @@ typedef struct ti113x_state_t {
     { "TI 1211",  IS_TI|IS_CARDBUS, ID(TI, 1211) }, \
     { "TI 1420",  IS_TI|IS_CARDBUS, ID(TI, 1420) }, \
     { "TI 1031",  IS_TI|IS_CARDBUS, ID(TI, 1031) }, \
-    { "TI 1410",  IS_TI|IS_CARDBUS, ID(TI, 1410) }
+    { "TI 1410",  IS_TI|IS_CARDBUS, ID(TI, 1410) }, \
+    { "TI 4451",  IS_TI|IS_CARDBUS, ID(TI, 4451) }
 
 #endif /* _LINUX_TI113X_H */
