@@ -30,6 +30,7 @@
 #endif
 
 #include <linux/config.h>
+#include <asm/segment.h>
 #ifdef CONFIG_MODVERSIONS
 #define MODVERSIONS
 #include <linux/modversions.h>
@@ -76,7 +77,7 @@ static struct {
    of statistics in the /proc filesystem */
 
 #define IGNLABEL 0&(int)
-char *statsLabels[] = {
+static char *statsLabels[] = {
 	"RxOverrun",
 	IGNLABEL "RxPlcpCrcErr",
 	IGNLABEL "RxPlcpFormatErr",
@@ -190,19 +191,45 @@ static char *ssids[3] = {0,0,0};
 static int io[4]={ 0,};
 static int irq[4]={ 0,};
 
-static int auto_wep = 0; /* If set, it tries to figure out the wep mode */
-static int aux_bap = 0; /* Checks to see if the aux ports are needed to read
-                           the bap, needed on some older cards and buses. */   
+static
+int maxencrypt = 0; /* The highest rate that the card can encrypt at.
+		       0 means no limit.  For old cards this was 4 */
+
+static
+int auto_wep = 0; /* If set, it tries to figure out the wep mode */
+static
+int aux_bap = 0; /* Checks to see if the aux ports are needed to read
+		    the bap, needed on some older cards and buses. */   
+static
+int adhoc = 0;
 
 #if (LINUX_VERSION_CODE > 0x20155)
 /* new kernel */
+MODULE_AUTHOR("Benjamin Reed");
+MODULE_DESCRIPTION("Support for Cisco/Aironet 802.11 wireless ethernet \
+                   cards.  Direct support for ISA/PCI cards and support \
+		   for PCMCIA when used with airo_cs.");
+MODULE_SUPPORTED_DEVICE("Aironet 4500, 4800 and Cisco 340");
 MODULE_PARM(io,"1-4i");
 MODULE_PARM(irq,"1-4i");
 MODULE_PARM(basic_rate,"i");
 MODULE_PARM(rates,"1-8i");
 MODULE_PARM(ssids,"1-3s");
 MODULE_PARM(auto_wep,"i");
+MODULE_PARM_DESC(auto_wep, "If non-zero, the driver will keep looping through \
+the authentication options until an association is made.  The value of \
+auto_wep is number of the wep keys to check.  A value of 2 will try using \
+the key at index 0 and index 1.");
 MODULE_PARM(aux_bap,"i");
+MODULE_PARM_DESC(aux_bap, "If non-zero, the driver will switch into a mode \
+than seems to work better for older cards with some older buses.  Before \
+switching it checks that the switch is needed.");
+MODULE_PARM(maxencrypt, "i");
+MODULE_PARM_DESC(maxencrypt, "The maximum speed that the card can do \
+encryption.  Units are in 512kbs.  Zero (default) means there is no limit. \
+Older cards used to be limited to 2mbs (4).");
+MODULE_PARM(adhoc, "i");
+MODULE_PARM_DESC(adhoc, "If non-zero, the card will start in adhoc mode.");
 
 #include <asm/uaccess.h>
 
@@ -292,8 +319,6 @@ static int do8bitIO = 0;
 #define BAP_ERR 0x4000
 #define BAP_DONE 0x2000
 
-#define PROMISC 0xffff
-
 #define EV_CMD 0x10
 #define EV_CLEARCOMMANDBUSY 0x4000
 #define EV_RX 0x01
@@ -302,12 +327,14 @@ static int do8bitIO = 0;
 #define EV_ALLOC 0x08
 #define EV_LINK 0x80
 #define EV_AWAKE 0x100
-#define STATUS_INTS ( EV_AWAKE | EV_LINK | EV_TXEXC | EV_TX | EV_RX )
+#define EV_UNKNOWN 0x800
+#define STATUS_INTS ( EV_AWAKE | EV_LINK | EV_TXEXC | EV_TX | EV_RX | EV_UNKNOWN)
 
 /* The RIDs */
 #define RID_WEP_PERM 0xFF16
 #define RID_WEP_TEMP 0xFF15
 #define RID_SSID     0xFF11
+#define RID_APLIST   0xFF12
 #define RID_CONFIG   0xFF10
 #define RID_ACTUALCONFIG 0xFF20 /*readonly*/
 #define RID_MODULATION 0xFF17
@@ -494,6 +521,11 @@ typedef struct {
 	u16 normalizedSignalStrength;
 } StatusRid;
 
+typedef struct {
+	u16 len;
+	u8 ap[4][6];
+} APListRid;
+
 typedef struct {
 	u16 len;
 	char oui[3];
@@ -532,7 +564,7 @@ typedef struct {
 #define BUSY_FID 0x10000
 
 static char *version =
-"airo.c 0.99zz 2000/05/26 09:40:55 (Benjamin Reed)";
+"airo.c 1.2 2000/10/19 14:12:32 (Benjamin Reed)";
 
 struct airo_info;
 
@@ -575,7 +607,9 @@ struct airo_info {
 	int                           fids[MAX_FIDS];
 	int registered;
 	ConfigRid config;
-	u16 authtype;
+	u16 authtype; // Used with auto_wep 
+	char keyindex; // Used with auto wep
+	char defindex; // Used with auto wep
 	struct timer_list timer;
 #if (LINUX_VERSION_CODE < 0x20311)
 	struct proc_dir_entry proc_entry;
@@ -584,6 +618,7 @@ struct airo_info {
 	struct proc_dir_entry proc_status_entry;
 	struct proc_dir_entry proc_config_entry;
 	struct proc_dir_entry proc_SSID_entry;
+	struct proc_dir_entry proc_APList_entry;
 	struct proc_dir_entry proc_wepkey_entry;
 #else
 	struct proc_dir_entry *proc_entry;
@@ -594,7 +629,6 @@ struct airo_info {
         spinlock_t aux_lock;
         spinlock_t cmd_lock;
         int flags;
-#define FLAG_PROMISC 0x01
 #define FLAG_RADIO_OFF 0x02
 	int (*bap_read)(struct airo_info*, u16 *pu16Dst, int bytelen, 
 		int whichbap);
@@ -622,7 +656,6 @@ static int airo_open(struct net_device *dev) {
 
 	netif_start_queue(dev);
 	netif_mark_up(dev);
-	
 	return 0;
 }
 
@@ -677,12 +710,7 @@ static struct net_device_stats *airo_get_stats(struct net_device *dev) {
 }
 
 static void airo_set_multicast_list(struct net_device *dev) {
-        struct airo_info *ai = (struct airo_info*)dev->priv;
-
-        ai->flags &= ~FLAG_PROMISC;
-	if (dev->flags&IFF_PROMISC) {
-	        ai->flags |= FLAG_PROMISC;
-	} else if ((dev->flags&IFF_ALLMULTI)||dev->mc_count>0) {
+	if ((dev->flags&IFF_ALLMULTI)||dev->mc_count>0) {
 		/* Turn on multicast.  (Should be already setup...) */
 	}
 }
@@ -705,7 +733,7 @@ static int airo_close(struct net_device *dev) {
 	struct airo_info *ai = (struct airo_info*)dev->priv;
 	ai->open--;
 	netif_stop_queue(dev);
-	netif_mark_down(dev);
+        netif_mark_down(dev);
 	if ( !ai->open ) {
 		disable_interrupts( ai );
 	}
@@ -727,7 +755,9 @@ void stop_airo_card( struct net_device *dev )
 	release_region( dev->base_addr, 64 );
 	free_irq( dev->irq, dev );
 	del_airo_dev( dev );
-	if (dev->priv)
+        /* This goofy + 1 thing is because sometimes init_ethernet will
+           do a separate allocation for priv and sometimes not... */
+	if (dev->priv && dev->priv != dev + 1)
 		kfree(dev->priv);
 	kfree( dev );
 	if (auto_wep) del_timer(&(ai)->timer);
@@ -742,13 +772,13 @@ struct net_device *init_airo_card( unsigned short irq, int port )
 	int i;
 	
 	/* Create the network device object. */
-	dev = kmalloc(sizeof(struct net_device), GFP_KERNEL);
-	memset(dev, 0, sizeof(struct net_device));
-	
-	/* Space for the info structure. */
-	dev->priv = kmalloc(sizeof(struct airo_info), GFP_KERNEL);
-	memset(dev->priv, 0, sizeof(struct airo_info));
+        dev = init_etherdev(0, sizeof(*ai));
+        if (!dev) {
+            printk(KERN_ERR "airo:  Couldn't init_etherdev\n");
+            return NULL;
+        }
 	ai = (struct airo_info *)dev->priv;
+	ai->registered = 1;
         ai->dev = dev;
 	ai->bap0_lock = SPIN_LOCK_UNLOCKED;
 	ai->bap1_lock = SPIN_LOCK_UNLOCKED;
@@ -761,10 +791,6 @@ struct net_device *init_airo_card( unsigned short irq, int port )
 	dev->get_stats = &airo_get_stats;
 	dev->set_multicast_list = &airo_set_multicast_list;
 	dev->do_ioctl = &private_ioctl;
-#if (LINUX_VERSION_CODE < 0x020363)
-	dev->name = ((struct airo_info *)dev->priv)->name;
-#endif
-	ether_setup(dev);
 	dev->change_mtu = &airo_change_mtu;
 	dev->open = &airo_open;
 	dev->stop = &airo_close;
@@ -785,7 +811,6 @@ struct net_device *init_airo_card( unsigned short irq, int port )
 			 &((struct airo_info*)dev->priv)->config) 
 	     != SUCCESS ) {
 		printk( KERN_ERR "airo: MAC could not be enabled\n" );
-		goto init_undo;
 	} else {
 		printk( KERN_INFO "airo: MAC enabled %s %x:%x:%x:%x:%x:%x\n",
 			dev->name,
@@ -796,25 +821,15 @@ struct net_device *init_airo_card( unsigned short irq, int port )
 			dev->dev_addr[4],
 			dev->dev_addr[5]
 			);
-		/* Allocate the transmit buffers */
-		for( i = 0; i < MAX_FIDS; i++ ) {
-			ai->fids[i] = transmit_allocate( ai, 2000 );
-		}
 	}
-	
-	if (register_netdev(dev) != 0) {
-		printk(KERN_ERR "airo: register_netdev() failed\n");
-		goto init_undo;
+	/* Allocate the transmit buffers */
+	for( i = 0; i < MAX_FIDS; i++ ) {
+		ai->fids[i] = transmit_allocate( ai, 2000 );
 	}
-	((struct airo_info*)dev->priv)->registered = 1;
 	
 	setup_proc_entry( dev, (struct airo_info*)dev->priv );
-	
 	netif_start_queue(dev);
 	return dev;
- init_undo:
-	stop_airo_card( dev );
-	return 0;
 }
 
 int reset_airo_card( struct net_device *dev ) {
@@ -874,7 +889,7 @@ static void airo_interrupt ( int irq, void* dev_id, struct pt_regs *regs) {
 		   monitor hook in, do it here.  (Remember that
 		   interrupts are still disabled!)
 		*/
-		/*u16 newStatus = */IN4500(apriv, LINKSTAT);
+		u16 newStatus = IN4500(apriv, LINKSTAT);
 		/* Here is what newStatus means: */
 #define NOBEACON 0x8000 /* Loss of sync - missed beacons */
 #define MAXRETRIES 0x8001 /* Loss of sync - max retries */
@@ -904,6 +919,12 @@ static void airo_interrupt ( int irq, void* dev_id, struct pt_regs *regs) {
 			  leaving BSS */
 #define RC_NOAUTH 9 /* Station requesting (Re)Association is not
 		       Authenticated with the responding station */
+		if (newStatus != ASSOCIATED) {
+			if (auto_wep && !timer_pending(&apriv->timer)) {
+				apriv->timer.expires = RUN_AT(HZ*3);
+	      			add_timer(&apriv->timer);
+			}
+		}
 	}
 	
 	/* Check to see if there is something to recieve */
@@ -1035,14 +1056,6 @@ static int enable_MAC( struct airo_info *ai, Resp *rsp ) {
 	memset(&cmd, 0, sizeof(cmd));
 	cmd.cmd = MAC_ENABLE; // disable in case already enabled
 	status = issuecommand(ai, &cmd, rsp);
-	if (status == SUCCESS && ai->flags|FLAG_PROMISC) {
-	        Cmd cmd;
-		Resp rsp;
-		memset(&cmd, 0, sizeof(cmd));
-		cmd.cmd=CMD_SETMODE;
-		cmd.parm1=PROMISC;
-		issuecommand(ai, &cmd, &rsp);
-	}
 	return status;
 }
 
@@ -1078,6 +1091,9 @@ static u16 setup_card(struct airo_info *ai, u8 *mac,
 	int status;
 	int i;
 	SsidRid mySsid;
+	u16 lastindex;
+	WepKeyRid wkr;
+	int rc;
 
 	memset( &mySsid, 0, sizeof( mySsid ) );
 
@@ -1112,7 +1128,7 @@ static u16 setup_card(struct airo_info *ai, u8 *mac,
 		// general configuration (read/modify/write)
 		status = PC4500_readrid(ai, RID_CONFIG, &cfg, sizeof(cfg));
 		if ( status != SUCCESS ) return ERROR;
-		cfg.opmode = MODE_STA_ESS; // station in ESS mode
+		cfg.opmode = adhoc ? MODE_STA_IBSS : MODE_STA_ESS;
     
 		/* Save off the MAC */
 		for( i = 0; i < 6; i++ ) {
@@ -1138,6 +1154,7 @@ static u16 setup_card(struct airo_info *ai, u8 *mac,
 				}
 			}
 		}
+		cfg.authType = ai->authtype;
 		*config = cfg;
 	}
 	
@@ -1161,7 +1178,21 @@ static u16 setup_card(struct airo_info *ai, u8 *mac,
 	status = PC4500_writerid(ai, RID_SSID, &mySsid, sizeof(mySsid));
 	if ( status != SUCCESS ) return ERROR;
 	
+	/* Grab the initial wep key, we gotta save it for auto_wep */
+	rc = PC4500_readrid(ai, RID_WEP_TEMP, &wkr, sizeof(wkr));
+	if (rc == SUCCESS) do {
+		lastindex = wkr.kindex;
+		if (wkr.kindex == 0xffff) {
+			ai->defindex = wkr.mac[0];
+		}
+	        PC4500_readrid(ai, RID_WEP_PERM, &wkr, sizeof(wkr));
+	} while(lastindex != wkr.kindex);
+	
 	status = enable_MAC(ai, &rsp);
+	if (auto_wep && !timer_pending(&ai->timer)) {
+		ai->timer.expires = RUN_AT(HZ*3);
+		add_timer(&ai->timer);
+	}
 	if (status != SUCCESS || (rsp.status & 0xFF00) != 0) {
 		int reason = rsp.rsp0;
 		int badRidNumber = rsp.rsp1;
@@ -1527,6 +1558,7 @@ static int proc_stats_open( struct inode *inode, struct file *file );
 static int proc_statsdelta_open( struct inode *inode, struct file *file );
 static int proc_status_open( struct inode *inode, struct file *file );
 static int proc_SSID_open( struct inode *inode, struct file *file );
+static int proc_APList_open( struct inode *inode, struct file *file );
 static int proc_config_open( struct inode *inode, struct file *file );
 static int proc_wepkey_open( struct inode *inode, struct file *file );
 
@@ -1555,6 +1587,13 @@ static struct file_operations proc_SSID_ops = {
 	release:       proc_close
 };
 
+static struct file_operations proc_APList_ops = {
+	read:          proc_read,
+	write:         proc_write,
+	open:          proc_APList_open,
+	release:       proc_close
+};
+
 static struct file_operations proc_config_ops = {
 	read:          proc_read,
 	write:         proc_write,
@@ -1582,6 +1621,9 @@ static struct inode_operations proc_inode_status_ops = {
 static struct inode_operations proc_inode_SSID_ops = {
 	&proc_SSID_ops};
 
+static struct inode_operations proc_inode_APList_ops = {
+	&proc_APList_ops};
+
 static struct inode_operations proc_inode_config_ops = {
 	&proc_config_ops};
 
@@ -1677,6 +1719,13 @@ static struct proc_dir_entry SSID_entry = {
 	&proc_inode_SSID_ops, NULL
 };
 
+static struct proc_dir_entry APList_entry = {
+	0, 6, "APList",
+	S_IFREG | S_IRUGO | S_IWUSR, 2, 0, 0,
+	13,
+	&proc_inode_APList_ops, NULL
+};
+
 static struct proc_dir_entry config_entry = {
 	0, 6, "Config",
 	S_IFREG | S_IRUGO | S_IWUSR, 2, 0, 0,
@@ -1741,6 +1790,13 @@ static int setup_proc_entry( struct net_device *dev,
 	PROC_REGISTER( &apriv->proc_entry, 
 		       &apriv->proc_SSID_entry );
 	
+	/* Setup the APList */
+	memcpy( &apriv->proc_APList_entry, &APList_entry, 
+		sizeof( APList_entry ) );
+	apriv->proc_APList_entry.data = dev;
+	PROC_REGISTER( &apriv->proc_entry, 
+		       &apriv->proc_APList_entry );
+	
 	/* Setup the WepKey */
 	memcpy( &apriv->proc_wepkey_entry, &wepkey_entry, 
 		sizeof( wepkey_entry ) );
@@ -1759,6 +1815,7 @@ static int takedown_proc_entry( struct net_device *dev,
 	PROC_UNREGISTER( &apriv->proc_entry, &apriv->proc_status_entry );
 	PROC_UNREGISTER( &apriv->proc_entry, &apriv->proc_config_entry );
 	PROC_UNREGISTER( &apriv->proc_entry, &apriv->proc_SSID_entry );
+	PROC_UNREGISTER( &apriv->proc_entry, &apriv->proc_APList_entry );
 	PROC_UNREGISTER( &apriv->proc_entry, &apriv->proc_wepkey_entry );
 	PROC_UNREGISTER( &airo_entry, &apriv->proc_entry );
 	return 0;
@@ -1810,6 +1867,13 @@ static int setup_proc_entry( struct net_device *dev,
 	entry->data = dev;
 	entry->proc_fops = &proc_SSID_ops;
 
+	/* Setup the APList */
+	entry = create_proc_entry("APList",
+				  S_IFREG | S_IRUGO | S_IWUGO,
+				  apriv->proc_entry);
+	entry->data = dev;
+	entry->proc_fops = &proc_APList_ops;
+
 	/* Setup the WepKey */
 	entry = create_proc_entry("WepKey",
 				  S_IFREG | S_IWUSR,
@@ -1828,6 +1892,7 @@ static int takedown_proc_entry( struct net_device *dev,
 	remove_proc_entry("Status",apriv->proc_entry);
 	remove_proc_entry("Config",apriv->proc_entry);
 	remove_proc_entry("SSID",apriv->proc_entry);
+	remove_proc_entry("APList",apriv->proc_entry);
 	remove_proc_entry("WepKey",apriv->proc_entry);
 	remove_proc_entry(dev->name,airo_entry);
 	return 0;
@@ -2053,6 +2118,18 @@ static int get_dec_u16( char *buffer, int *start, int limit ) {
 	return value;
 }
 
+static void checkThrottle(ConfigRid *config) {
+	int i;
+	/* Old hardware had a limit on encryption speed */
+	if (config->authType != AUTH_OPEN && maxencrypt) {
+		for(i=0; i<8; i++) {
+			if (config->rates[i] > maxencrypt) {
+				config->rates[i] = 0;
+			}
+		}
+	}
+}
+
 static void proc_config_on_close( struct inode *inode, struct file *file ) {
 	struct proc_data *data = file->private_data;
 	struct proc_dir_entry *dp = inode->u.generic_ip;
@@ -2219,6 +2296,7 @@ static void proc_config_on_close( struct inode *inode, struct file *file ) {
 		if ( line[0] ) line++;
 	}
 	ai->config = config;
+	checkThrottle(&config);
 	PC4500_writerid(ai, RID_CONFIG, &config,
 			sizeof(config));
 	enable_MAC(ai, &rsp);
@@ -2334,6 +2412,44 @@ static void proc_SSID_on_close( struct inode *inode, struct file *file ) {
 	do_writerid(ai, RID_SSID, &SSID_rid, sizeof( SSID_rid ));
 }
 
+inline static u8 hexVal(char c) {
+	if (c>='0' && c<='9') return c -= '0';
+	if (c>='a' && c<='f') return c -= 'a'-10;
+	if (c>='A' && c<='F') return c -= 'A'-10;
+	return 0;
+}
+
+static void proc_APList_on_close( struct inode *inode, struct file *file ) {
+	struct proc_data *data = (struct proc_data *)file->private_data;
+	struct proc_dir_entry *dp = inode->u.generic_ip;
+	struct net_device *dev = dp->data;
+	struct airo_info *ai = (struct airo_info*)dev->priv;
+	APListRid APList_rid;
+	int i;
+	
+	if ( !data->writelen ) return;
+	
+	memset( &APList_rid, 0, sizeof(APList_rid) );
+	APList_rid.len = sizeof(APList_rid);
+	
+	for( i = 0; i < 4 && data->writelen >= (i+1)*6*3; i++ ) {
+		int j;
+		for( j = 0; j < 6*3 && data->wbuffer[j+i*6*3]; j++ ) {
+			switch(j%3) {
+			case 0:
+				APList_rid.ap[i][j/3]=
+					hexVal(data->wbuffer[j+i*6*3])<<4;
+				break;
+			case 1:
+				APList_rid.ap[i][j/3]|=
+					hexVal(data->wbuffer[j+i*6*3]);
+				break;
+			}
+		}
+	}
+	do_writerid(ai, RID_APLIST, &APList_rid, sizeof( APList_rid ));
+}
+
 /* This function wraps PC4500_writerid with a MAC disable */
 static int do_writerid( struct airo_info *ai, u16 rid, const void *rid_data,
 			int len ) {
@@ -2350,27 +2466,37 @@ static int do_writerid( struct airo_info *ai, u16 rid, const void *rid_data,
 	return rc;
 }
 
-static int set_wep_key(struct airo_info *ai, const char *key, u16 keylen ) {
+static int set_wep_key(struct airo_info *ai, u16 index,
+		       const char *key, u16 keylen, int perm ) {
 	static const unsigned char macaddr[6] = { 0x01, 0, 0, 0, 0, 0 };
 	WepKeyRid wkr;
         int rc;
 
-	wkr.len = cpu_to_le16(sizeof(wkr));
-	wkr.kindex = cpu_to_le16(0);
-	wkr.klen = cpu_to_le16(keylen);
-	memcpy( wkr.key, key, keylen );
-	memcpy( wkr.mac, macaddr, 6 );
+	memset(&wkr, 0, sizeof(wkr));
+	if (keylen == 0) {
+		// We are selecting which key to use
+		wkr.len = cpu_to_le16(sizeof(wkr));
+		wkr.kindex = cpu_to_le16(0xffff);
+		wkr.mac[0] = (char)index;
+		if (perm) printk(KERN_INFO "Setting transmit key to %d\n", index);
+		if (perm) ai->defindex = (char)index;
+	} else {
+		// We are actually setting the key
+		wkr.len = cpu_to_le16(sizeof(wkr));
+		wkr.kindex = cpu_to_le16(index);
+		wkr.klen = cpu_to_le16(keylen);
+		memcpy( wkr.key, key, keylen );
+	        memcpy( wkr.mac, macaddr, 6 );
+		printk(KERN_INFO "Setting key %d\n", index);
+	}
 	rc = do_writerid(ai, RID_WEP_TEMP, &wkr, sizeof(wkr));
         if (rc!=SUCCESS) printk(KERN_ERR "airo:  WEP_TEMP set %x\n", rc); 
-	rc = do_writerid(ai, RID_WEP_PERM, &wkr, sizeof(wkr));
-        if (rc!=SUCCESS) printk(KERN_ERR "airo:  WEP_PERM set %x\n", rc);
-	return 0;
-}
-
-static u8 hexVal(char c) {
-	if (c>='0' && c<='9') return c -= '0';
-	if (c>='a' && c<='f') return c -= 'a'-10;
-	if (c>='A' && c<='F') return c -= 'A'-10;
+	if (perm) {
+		rc = do_writerid(ai, RID_WEP_PERM, &wkr, sizeof(wkr));
+		if (rc!=SUCCESS) {
+			printk(KERN_ERR "airo:  WEP_PERM set %x\n", rc);
+		}
+	}
 	return 0;
 }
 
@@ -2381,6 +2507,8 @@ static void proc_wepkey_on_close( struct inode *inode, struct file *file ) {
 	struct airo_info *ai = (struct airo_info*)dev->priv;
 	int i;
 	char key[16];
+	u16 index = 0;
+	int j = 0;
 
 	memset(key, 0, sizeof(key));
 	
@@ -2388,17 +2516,30 @@ static void proc_wepkey_on_close( struct inode *inode, struct file *file ) {
 	data = (struct proc_data *)file->private_data;
 	if ( !data->writelen ) return;
 	
-	for( i = 0; i < 16*3 && data->wbuffer[i]; i++ ) {
+	if (data->wbuffer[0] >= '0' && data->wbuffer[0] <= '3' &&
+	    (data->wbuffer[1] == ' ' || data->wbuffer[1] == '\n')) {
+		index = data->wbuffer[0] - '0';
+		if (data->wbuffer[1] == '\n') {
+			set_wep_key(ai, index, 0, 0, 1);
+			return;
+		}
+		j = 2;
+	} else {
+		printk(KERN_ERR "airo:  WepKey passed invalid key index\n");
+		return;
+	}
+
+	for( i = 0; i < 16*3 && data->wbuffer[i+j]; i++ ) {
 		switch(i%3) {
 		case 0:
-			key[i/3] = hexVal(data->wbuffer[i])<<4;
+			key[i/3] = hexVal(data->wbuffer[i+j])<<4;
 			break;
 		case 1:
-			key[i/3] |= hexVal(data->wbuffer[i]);
+			key[i/3] |= hexVal(data->wbuffer[i+j]);
 			break;
 		}
 	}
-	set_wep_key(ai, key, i/3);
+	set_wep_key(ai, index, key, i/3, 1);
 }
 
 static int proc_wepkey_open( struct inode *inode, struct file *file ) {
@@ -2408,6 +2549,9 @@ static int proc_wepkey_open( struct inode *inode, struct file *file ) {
 	struct airo_info *ai = (struct airo_info*)dev->priv;
 	char *ptr;
 	WepKeyRid wkr;
+	u16 lastindex;
+	int j=0;
+	int rc;
 	
 	MOD_INC_USE_COUNT;
 	
@@ -2417,16 +2561,29 @@ static int proc_wepkey_open( struct inode *inode, struct file *file ) {
 	memset(file->private_data, 0, sizeof(struct proc_data));
 	memset(&wkr, 0, sizeof(wkr));
 	data = (struct proc_data *)file->private_data;
-	data->rbuffer = kmalloc( 80, GFP_KERNEL );
+	data->rbuffer = kmalloc( 180, GFP_KERNEL );
+	memset(data->rbuffer, 0, 180);
 	data->writelen = 0;
 	data->maxwritelen = 80;
 	data->wbuffer = kmalloc( 80, GFP_KERNEL );
 	memset( data->wbuffer, 0, 80 );
 	data->on_close = proc_wepkey_on_close;
 	
-	PC4500_readrid(ai, RID_WEP_PERM, &wkr, sizeof(wkr));
 	ptr = data->rbuffer;
-	sprintf(ptr, "The wep key cannot be read\n");
+	strcpy(ptr, "No wep keys\n");
+	rc = PC4500_readrid(ai, RID_WEP_TEMP, &wkr, sizeof(wkr));
+	if (rc == SUCCESS) do {
+		lastindex = wkr.kindex;
+		if (wkr.kindex == 0xffff) {
+			j += sprintf(ptr+j, "Tx key = %d\n",
+			             (int)wkr.mac[0]);
+		} else {
+		        j += sprintf(ptr+j, "Key %d set with length = %d\n",
+                                     (int)wkr.kindex, (int)wkr.klen);
+                }
+	        PC4500_readrid(ai, RID_WEP_PERM, &wkr, sizeof(wkr));
+	} while((lastindex != wkr.kindex) && (j < 180-30));
+
 	data->readlen = strlen( data->rbuffer );
 	return 0;
 }
@@ -2472,6 +2629,51 @@ static int proc_SSID_open( struct inode *inode, struct file *file ) {
 	return 0;
 }
 
+static int proc_APList_open( struct inode *inode, struct file *file ) {
+	struct proc_data *data;
+	struct proc_dir_entry *dp = inode->u.generic_ip;
+	struct net_device *dev = dp->data;
+	struct airo_info *ai = (struct airo_info*)dev->priv;
+	int i;
+	char *ptr;
+	APListRid APList_rid;
+
+	MOD_INC_USE_COUNT;
+	
+	dp = (struct proc_dir_entry *) inode->u.generic_ip;
+	
+	file->private_data = kmalloc(sizeof(struct proc_data ), GFP_KERNEL);
+	memset(file->private_data, 0, sizeof(struct proc_data));
+	data = (struct proc_data *)file->private_data;
+	data->rbuffer = kmalloc( 104, GFP_KERNEL );
+	data->writelen = 0;
+	data->maxwritelen = 4*6*3;
+	data->wbuffer = kmalloc( data->maxwritelen, GFP_KERNEL );
+	memset( data->wbuffer, 0, data->maxwritelen );
+	data->on_close = proc_APList_on_close;
+	
+	PC4500_readrid(ai, RID_APLIST, 
+		       &APList_rid, sizeof(APList_rid));
+	ptr = data->rbuffer;
+	for( i = 0; i < 4; i++ ) {
+		// We end when we find a zero MAC
+		if ( !*(int*)APList_rid.ap[i] &&
+		     !*(int*)&APList_rid.ap[i][2]) break;
+		ptr += sprintf(ptr, "%02x:%02x:%02x:%02x:%02x:%02x\n",
+			       (int)APList_rid.ap[i][0],
+			       (int)APList_rid.ap[i][1],
+			       (int)APList_rid.ap[i][2],
+			       (int)APList_rid.ap[i][3],
+			       (int)APList_rid.ap[i][4],
+			       (int)APList_rid.ap[i][5]);
+	}
+        if (i==0) ptr += sprintf(ptr, "Not using specific APs\n");
+
+	*ptr = '\0';
+	data->readlen = strlen( data->rbuffer );
+	return 0;
+}
+
 #if (LINUX_VERSION_CODE > 0x20155)
 static int proc_close( struct inode *inode, struct file *file ) 
 #else
@@ -2505,49 +2707,41 @@ static void timer_func( u_long data ) {
 	u16 linkstat = IN4500(apriv, LINKSTAT);
 	
 	if (linkstat != 0x400 ) {
-		struct timer_list *timer = &apriv->timer;
+		/* We don't have a link so try changing the authtype */
 		ConfigRid config = apriv->config;
-		int i;
-		
+
 		switch(apriv->authtype) {
 		case AUTH_ENCRYPT:
-			/* So escalate to SHAREDKEY */
-			config.authType = AUTH_SHAREDKEY;
-			/* The current hardware can't do more than
-			   2mbs with encryption */
-			for(i=0; i<8; i++) {
-				if (config.rates[i] > 4) {
-					config.rates[i] = 0;
-				}
-			}
-			apriv->authtype = AUTH_SHAREDKEY;
-			break;
-		case AUTH_SHAREDKEY:
-			/* Drop to open */
+			/* So drop to OPEN */
 			config.authType = AUTH_OPEN;
 			apriv->authtype = AUTH_OPEN;
 			break;
-		default:  /* We'll default to open */
-			/* So escalate to ENCRYPT */
-			config.authType = AUTH_ENCRYPT;
-			/* The current hardware can't do more than
-			   2mbs with encryption */
-			for(i=0; i<8; i++) {
-				if (config.rates[i] > 4) {
-					config.rates[i] = 0;
-				}
+		case AUTH_SHAREDKEY:
+			if (apriv->keyindex < auto_wep) {
+				set_wep_key(apriv, apriv->keyindex, 0, 0, 0);
+				config.authType = AUTH_SHAREDKEY;
+				apriv->authtype = AUTH_SHAREDKEY;
+			        apriv->keyindex++;
+			} else {
+			        /* Drop to ENCRYPT */
+			        apriv->keyindex = 0;
+			        set_wep_key(apriv, apriv->defindex, 0, 0, 0);
+			        config.authType = AUTH_ENCRYPT;
+			        apriv->authtype = AUTH_ENCRYPT;
 			}
-			apriv->authtype = AUTH_ENCRYPT;
+			break;
+		default:  /* We'll escalate to SHAREDKEY */
+			config.authType = AUTH_SHAREDKEY;
+			apriv->authtype = AUTH_SHAREDKEY;
+		}
+		checkThrottle(&config);
+		do_writerid(apriv, RID_CONFIG, &config, sizeof(config));
+		if (!timer_pending(&apriv->timer)) {
+			/* Schedule check to see if the change worked */
+			apriv->timer.expires = RUN_AT(HZ*3);
+	      		add_timer(&apriv->timer);
 		}
-		setup_card(apriv, dev->dev_addr, &config);
-		timer->expires = RUN_AT(HZ*5);  /*Check every 5 secs
-						  until everything is OK*/
-	} else {
-		struct timer_list *timer = &apriv->timer;
-		timer->expires = RUN_AT(HZ*60);  /*Check every 60 secs
-						   until everything is OK*/
 	}
-	add_timer(&apriv->timer);
 }
 
 static void add_airo_dev( struct net_device *dev ) {
@@ -2557,15 +2751,14 @@ static void add_airo_dev( struct net_device *dev ) {
 		printk( KERN_ERR "airo_pci:  Out of memory\n" );
 	} else {
 		if ( auto_wep ) {
+	                struct airo_info *apriv=(struct airo_info *)dev->priv;
 			struct timer_list *timer = 
 				&((struct airo_info*)dev->priv)->timer;
 			
 			timer->function = timer_func;
 			timer->data = (u_long)dev;
 			init_timer(timer);
-			/*Start off checking 5 secs */
-			timer->expires = RUN_AT( HZ * 5 );  
-			add_timer(timer);
+			apriv->authtype = AUTH_SHAREDKEY;
 		}
 		
 		node->dev = dev;
