@@ -6,7 +6,7 @@
  * Status:        Experimental.
  * Authors:       John Markus Bjrndalen <johnm@cs.uit.no>
  *                Dag Brattli <dagb@cs.uit.no>
- *                David Hinds <dhinds@pcmcia.sourceforge.org>
+ *                David Hinds <dahinds@users.sourceforge.net>
  * Created at:    A long time ago!
  * Modified at:   Mon Nov 10 11:54:37 1997
  * Modified by:   Dag Brattli <dagb@cs.uit.no>
@@ -238,7 +238,7 @@ static void netwave_watchdog(u_long);
 
 /* Statistics */
 static void update_stats(struct net_device *dev);
-static struct enet_statistics *netwave_get_stats(struct net_device *dev);
+static struct net_device_stats *netwave_get_stats(struct net_device *dev);
 
 /* Wireless extensions */
 #ifdef WIRELESS_EXT
@@ -314,14 +314,14 @@ typedef struct netwave_private {
     int        lastExec;
     struct timer_list      watchdog;	/* To avoid blocking state */
     struct site_survey     nss;
-    struct enet_statistics stats;
+    struct net_device_stats stats;
 #ifdef WIRELESS_EXT
     struct iw_statistics   iw_stats;    /* Wireless stats */
 #endif
 } netwave_private;
 
 #ifdef NETWAVE_STATS
-static struct enet_statistics *netwave_get_stats(struct net_device *dev);
+static struct net_device_stats *netwave_get_stats(struct net_device *dev);
 #endif
 
 /*
@@ -852,7 +852,7 @@ static void netwave_pcmcia_config(dev_link_t *link) {
      *  If you want, you can also read the card's config table to
      *  pick addresses -- see the serial driver for an example.
      */
-    for (j = 0x0; j < 0x400; j += 0x20) {
+    for (i = j = 0x0; j < 0x400; j += 0x20) {
 	link->io.BasePort1 = j ^ 0x300;
 	i = CardServices(RequestIO, link->handle, &link->io);
 	if (i == CS_SUCCESS) break;
@@ -1151,7 +1151,7 @@ static int netwave_hw_xmit(unsigned char* data, int len,
     netwave_private *priv = (netwave_private *) dev->priv;
     u_char* ramBase = priv->ramBase;
     ioaddr_t iobase = dev->base_addr;
-	
+
     /* Disable interrupts & save flags */
     save_flags(flags);
     cli();
@@ -1162,6 +1162,7 @@ static int netwave_hw_xmit(unsigned char* data, int len,
 	/* No buffers available */
 	printk(KERN_DEBUG "netwave_hw_xmit: %s - no xmit buffers available.\n",
 	       dev->name);
+	restore_flags(flags);
 	return 1;
     }
 
@@ -1402,7 +1403,7 @@ static void netwave_watchdog(u_long a) {
     netif_start_queue(dev);
 } /* netwave_watchdog */
 
-static struct enet_statistics *netwave_get_stats(struct net_device *dev) {
+static struct net_device_stats *netwave_get_stats(struct net_device *dev) {
     netwave_private *priv = (netwave_private*)dev->priv;
 
     update_stats(dev);
