@@ -36,9 +36,9 @@
  *
  * SMP
  * ---
- *	We now *should* be SMP compliant.
- *	I don't have a SMP box to verify that (my Pentium 133 is not), so
- *	someone has to certify that from me.
+ *	We now are SMP compliant (I eventually fixed the remaining bugs).
+ *	The driver has been tested on a dual P6-150 and survived my usual
+ *	set of torture tests.
  *	Anyway, I spent enough time chasing interrupt re-entrancy during
  *	errors or reconfigure, and I designed the locked/unlocked sections
  *	of the driver with great care, and with the recent addition of
@@ -141,7 +141,7 @@
  * Yunzhou Li <yunzhou@strat.iol.unh.edu> finished is work.
  * Joe Finney <joe@comp.lancs.ac.uk> patched the driver to start
  * correctly 2.00 cards (2.4 GHz with frequency selection).
- * David Hinds <dhinds@pcmcia.sourceforge.org> integrated the whole in his
+ * David Hinds <dahinds@users.sourceforge.net> integrated the whole in his
  * Pcmcia package (+ bug corrections).
  *
  * I (Jean Tourrilhes - jt@hplb.hpl.hp.com) then started to make some
@@ -193,7 +193,7 @@
  *	Matthew Geier (matthew@cs.su.oz.au),
  *	Remo di Giovanni (remo@cs.su.oz.au),
  *	Mark Hagan (mhagan@wtcpost.daytonoh.NCR.COM),
- *	David Hinds <dhinds@pcmcia.sourceforge.org>,
+ *	David Hinds <dahinds@users.sourceforge.net>,
  *	Jan Hoogendoorn (c/o marteijn@lucent.com),
  *      Bruce Janson <bruce@cs.usyd.edu.au>,
  *	Anthony D. Joseph <adj@lcs.mit.edu>,
@@ -384,6 +384,10 @@
  *	- Re-sync kernel/pcmcia versions (not much actually)
  *	- A few other cleanups (David & me)...
  *
+ * Changes made for release in 3.1.22 :
+ * ----------------------------------
+ *	- Check that SMP works, remove annoying log message
+ *
  * Wishes & dreams:
  * ----------------
  *	- Cleanup and integrate the roaming code
@@ -504,7 +508,7 @@
 /************************ CONSTANTS & MACROS ************************/
 
 #ifdef DEBUG_VERSION_SHOW
-static const char *version = "wavelan_cs.c : v22 (wireless extensions) 23/02/00\n";
+static const char *version = "wavelan_cs.c : v23 (wireless extensions) 10/10/00\n";
 #endif
 
 /* Watchdog temporisation */
